--
--	Copyright Cypress Semiconductor Corporation, 1994,1995,1996
--	  as an unpublished work.
--

use work.cypress.all;  
PACKAGE mcpartspkg IS

component PAincell		-- Master Input Cell.
    PORT (
	ip	: IN BIT;	-- Input pad.
	ini	: OUT BIT;	-- Inverting output of cell.
	iz	: OUT BIT	-- Non-inverting output of cell.
    );
end component;
    
component PAbicell		-- Master Bi-directional Cell.
    PORT (
	ie	: IN BIT;	-- Output Enable.
	i1	: IN BIT;	-- Inverting input.
	i2	: IN BIT;	-- Non-inverting input.
	iz	: OUT BIT;	-- Path into chip.
	ip	: INOUT x01z	-- I/O pad.
    );
end component;
	
component PAckcell		-- Master Clock Cell.
    PORT (
	ip	: IN BIT;	-- Input pad.
	ini	: OUT BIT;	-- Inverted.
	iz	: OUT BIT;	-- Non-inverted.
	ic	: OUT BIT	-- To "Dedicated clock network".
    );
end component;

component PAincel2		-- Master Input Cell.
    PORT (
	ip	: IN BIT;	-- Input pad.
	ini	: OUT BIT;	-- Inverting output of cell.
	iz	: OUT BIT;	-- Non-inverting output of cell.
	iqe	: IN BIT;	-- FF enable.
	iqc	: IN BIT;	-- FF clock.
	ir	: IN BIT;	-- FF reset.
	iq	: OUT BIT	-- FF output.
    );
END component;
    
component PAbicel2		-- Master Bi-directional Cell.
    PORT (
	ie	: IN BIT;	-- Output Enable.
	i1	: IN BIT;	-- Inverting input.
	i2	: IN BIT;	-- Non-inverting input.
	iz	: OUT BIT;	-- Path into chip.
	ip	: INOUT x01z;	-- I/O pad.
	iqe	: IN BIT;	-- FF enable.
	iqc	: IN BIT;	-- FF clock.
	ir	: IN BIT;	-- FF reset.
	iq	: OUT BIT	-- FF output.
    );
END component;
	
component PAckcel2		-- Master Clock Cell.
    PORT (
	ip	: IN BIT;	-- Input pad.
	ini	: OUT BIT;	-- Inverted.
	iz	: OUT BIT;	-- Non-inverted.
	ic	: OUT BIT;	-- To "Dedicated clock network".
	iqe	: IN BIT;	-- FF enable.
	iqc	: IN BIT;	-- FF clock.
	ir	: IN BIT;	-- FF reset.
	iq	: OUT BIT	-- FF output.
    );
END component;

component clkbuf		-- Clock Buffer.
    PORT (
	a	: IN BIT;	-- Input.
	z	: OUT BIT	-- Output.
    );
END component;

--		Fragment Cells:

component PAfrag_a
    PORT (
	a1	: IN BIT;	-- Non-invert to AND A.
	a2	: IN BIT;	-- Invert to AND A.
	a3	: IN BIT;	-- Non-invert to AND A.
	a4	: IN BIT;	-- Invert to AND A.
	a5	: IN BIT;	-- Non-invert to AND A.
	a6	: IN BIT;	-- Invert to AND A.

	az	: OUT BIT	-- Output of AND A.
    );
end component;
    
component PAfrag_f		-- Fragment AND F.
    PORT (
	f1	: IN BIT;	-- Non-invert to AND F.
	f2	: IN BIT;	-- Invert to AND F.
	f3	: IN BIT;	-- Non-invert to AND F.
	f4	: IN BIT;	-- Invert to AND F.
	f5	: IN BIT;	-- Non-invert to AND F.
	f6	: IN BIT;	-- Invert to AND F.
	fz	: OUT BIT	-- Output of AND F.
    );
end component;
    
component PAfrag_m		-- Fragment M.
    PORT (
	os	: IN BIT;	-- Select to MUX O.
	nsi	: IN BIT;	-- Select to MUX N.
	b1	: IN BIT;	-- Non-invert to AND B.
	b2	: IN BIT;	-- Invert to AND B.
	c1	: IN BIT;	-- Non-invert to AND C.
	c2	: IN BIT;	-- Invert to AND C.
	d1	: IN BIT;	-- Non-invert to AND D.
	d2	: IN BIT;	-- Invert to AND D.
	e1	: IN BIT;	-- Non-invert to AND E.
	e2	: IN BIT;	-- Invert to AND E.
	oz	: OUT BIT;	-- Output of MUX O.
	nz	: OUT BIT	-- Output of MUX N.
    );
end component;

component PAfrag_q		-- Fragment Q (flip flop).
    PORT (
	qs	: IN BIT;	-- Async Set on FF(Q).
	qc	: IN BIT;	-- Clock to FF(Q).
	qr	: IN BIT;	-- Async Reset on FF(Q).
	qd	: IN BIT;	-- Data on FF(Q).
	qz	: OUT BIT	-- Output of FF  Q.
    );
end component;

component PAlcell
    PORT (
	qs	: IN BIT;	-- Async Set on FF(Q).

	a1	: IN BIT;	-- Non-invert to AND A.
	a2	: IN BIT;	-- Invert to AND A.
	a3	: IN BIT;	-- Non-invert to AND A.
	a4	: IN BIT;	-- Invert to AND A.
	a5	: IN BIT;	-- Non-invert to AND A.
	a6	: IN BIT;	-- Invert to AND A.

	b1	: IN BIT;	-- Non-invert to AND B.
	b2	: IN BIT;	-- Invert to AND B.

	c1	: IN BIT;	-- Non-invert to AND C.
	c2	: IN BIT;	-- Invert to AND C.

	d1	: IN BIT;	-- Non-invert to AND D.
	d2	: IN BIT;	-- Invert to AND D.

	e1	: IN BIT;	-- Non-invert to AND E.
	e2	: IN BIT;	-- Invert to AND E.

	f1	: IN BIT;	-- Non-invert to AND F.
	f2	: IN BIT;	-- Invert to AND F.
	f3	: IN BIT;	-- Non-invert to AND F.
	f4	: IN BIT;	-- Invert to AND F.
	f5	: IN BIT;	-- Non-invert to AND F.
	f6	: IN BIT;	-- Invert to AND F.

	qc	: IN BIT;	-- Clock to FF(Q).
	qr	: IN BIT;	-- Async Reset on FF(Q).

	az	: OUT BIT;	-- Output of AND A.
	oz	: OUT BIT;	-- Output of MUX O.
	qz	: OUT BIT;	-- Output of FF  Q.
	nz	: OUT BIT;	-- Output of MUX N.
	fz	: OUT BIT	-- Output of AND F.
    );
end component;

component PAlcell2
    PORT (
	qs	: IN BIT;	-- Async Set on FF(Q).

	a1	: IN BIT;	-- Non-invert to AND A.
	a2	: IN BIT;	-- Invert to AND A.
	a3	: IN BIT;	-- Non-invert to AND A.
	a4	: IN BIT;	-- Invert to AND A.
	a5	: IN BIT;	-- Non-invert to AND A.
	a6	: IN BIT;	-- Invert to AND A.

	os	: IN BIT;       -- Data input MUX O.
	op	: IN BIT;       -- Select input MUX O.

	b1	: IN BIT;	-- Non-invert to AND B.
	b2	: IN BIT;	-- Invert to AND B.

	c1	: IN BIT;	-- Non-invert to AND C.
	c2	: IN BIT;	-- Invert to AND C.
	 
	mp	: IN BIT;       -- Select input MUX M.
	ms	: IN BIT;       -- Data input MUX M.
		      
	d1	: IN BIT;	-- Non-invert to AND D.
	d2	: IN BIT;	-- Invert to AND D.

	e1	: IN BIT;	-- Non-invert to AND E.
	e2	: IN BIT;	-- Invert to AND E.

	np	: IN BIT;       -- Select input MUX N.
	ns	: IN BIT;       -- Data input MUX N.
		     
	f1	: IN BIT;	-- Non-invert to AND F.
	f2	: IN BIT;	-- Invert to AND F.
	f3	: IN BIT;	-- Non-invert to AND F.
	f4	: IN BIT;	-- Invert to AND F.
	f5	: IN BIT;	-- Non-invert to AND F.
	f6	: IN BIT;	-- Invert to AND F.

	qc	: IN BIT;	-- Clock to FF(Q).
	qr	: IN BIT;	-- Async Reset on FF(Q).

	az	: OUT BIT;	-- Output of AND A.
	oz	: OUT BIT;	-- Output of MUX O.
	qz	: OUT BIT;	-- Output of FF  Q.
	nz	: OUT BIT;	-- Output of MUX N.
	fz	: OUT BIT	-- Output of AND F.
    );
end component;

component PPAlcell
    PORT (
	qs	: IN BIT;	-- Async Set on FF(Q).

	a1	: IN BIT;	-- Non-invert to AND A.
	a2	: IN BIT;	-- Invert to AND A.
	a3	: IN BIT;	-- Non-invert to AND A.
	a4	: IN BIT;	-- Invert to AND A.
	a5	: IN BIT;	-- Non-invert to AND A.
	a6	: IN BIT;	-- Invert to AND A.

	b1	: IN BIT;	-- Non-invert to AND B.
	b2	: IN BIT;	-- Invert to AND B.

	c1	: IN BIT;	-- Non-invert to AND C.
	c2	: IN BIT;	-- Invert to AND C.

	d1	: IN BIT;	-- Non-invert to AND D.
	d2	: IN BIT;	-- Invert to AND D.

	e1	: IN BIT;	-- Non-invert to AND E.
	e2	: IN BIT;	-- Invert to AND E.

	f1	: IN BIT;	-- Non-invert to AND F.
	f2	: IN BIT;	-- Invert to AND F.
	f3	: IN BIT;	-- Non-invert to AND F.
	f4	: IN BIT;	-- Invert to AND F.
	f5	: IN BIT;	-- Non-invert to AND F.
	f6	: IN BIT;	-- Invert to AND F.

	qc	: IN BIT;	-- Clock to FF(Q).
	qr	: IN BIT;	-- Async Reset on FF(Q).

	az	: OUT BIT;	-- Output of AND A.
	oz	: OUT BIT;	-- Output of MUX O.
	qz	: OUT BIT;	-- Output of FF  Q.
	nz	: OUT BIT;	-- Output of MUX N.
	fz	: OUT BIT	-- Output of AND F.
    );
end component;

component PAlogic
    PORT (
	qs	: IN BIT;	-- Async Set on FF(Q).

	a1	: IN BIT;	-- Non-invert to AND A.
	a2	: IN BIT;	-- Invert to AND A.
	a3	: IN BIT;	-- Non-invert to AND A.
	a4	: IN BIT;	-- Invert to AND A.
	a5	: IN BIT;	-- Non-invert to AND A.
	a6	: IN BIT;	-- Invert to AND A.

	b1	: IN BIT;	-- Non-invert to AND B.
	b2	: IN BIT;	-- Invert to AND B.

	c1	: IN BIT;	-- Non-invert to AND C.
	c2	: IN BIT;	-- Invert to AND C.

	d1	: IN BIT;	-- Non-invert to AND D.
	d2	: IN BIT;	-- Invert to AND D.

	e1	: IN BIT;	-- Non-invert to AND E.
	e2	: IN BIT;	-- Invert to AND E.

	f1	: IN BIT;	-- Non-invert to AND F.
	f2	: IN BIT;	-- Invert to AND F.
	f3	: IN BIT;	-- Non-invert to AND F.
	f4	: IN BIT;	-- Invert to AND F.
	f5	: IN BIT;	-- Non-invert to AND F.
	f6	: IN BIT;	-- Invert to AND F.

	qc	: IN BIT;	-- Clock to FF(Q).
	qr	: IN BIT;	-- Async Reset on FF(Q).

	az	: OUT BIT;	-- Output of AND A.
	oz	: OUT BIT;	-- Output of MUX O.
	qz	: OUT BIT;	-- Output of FF  Q.
	nz	: OUT BIT;	-- Output of MUX N.
	fz	: OUT BIT	-- Output of AND F.
    );
end component;

component logico
    PORT (
	a1	: IN BIT;	-- Non-invert to AND A.
	a2	: IN BIT;	-- Invert to AND A.
	a3	: IN BIT;	-- Non-invert to AND A.
	a4	: IN BIT;	-- Invert to AND A.
	a5	: IN BIT;	-- Non-invert to AND A.
	a6	: IN BIT;	-- Invert to AND A.

	b1	: IN BIT;	-- Non-invert to AND B.
	b2	: IN BIT;	-- Invert to AND B.

	c1	: IN BIT;	-- Non-invert to AND C.
	c2	: IN BIT;	-- Invert to AND C.

	d1	: IN BIT;	-- Non-invert to AND D.
	d2	: IN BIT;	-- Invert to AND D.

	e1	: IN BIT;	-- Non-invert to AND E.
	e2	: IN BIT;	-- Invert to AND E.

	f1	: IN BIT;	-- Non-invert to AND F.
	f2	: IN BIT;	-- Invert to AND F.
	f3	: IN BIT;	-- Non-invert to AND F.
	f4	: IN BIT;	-- Invert to AND F.
	f5	: IN BIT;	-- Non-invert to AND F.
	f6	: IN BIT;	-- Invert to AND F.

	oz	: OUT BIT	-- Output of MUX O.
    );
end component;

end mcpartspkg;

package mcpartspkg_local is
component buf
    port(
	x	: in bit;		-- Input to Buffer.
	y	: out bit		-- Output from Buffer.
    );
end component;
component bufoe
    port(
	x	: in bit;		-- Logic input to Buffer.
	oe	: in bit;		-- Output Enable input.
	y	: inout x01z;		-- Output pin.
	yfb	: out bit		-- Output feed back as bit.
    );
end component;
component dsrff
    port(
	d	: in bit;		-- D input to register.
	s	: in bit;		-- Set input Async.
	r	: in bit;		-- Reset input Async.
	clk	: in bit;		-- Positive edge Clock.
	q	: out bit		-- Output from register.
    );
end component;
end mcpartspkg_local ;

--	pASIC cells maped to this device.

ENTITY PAincell is		-- Master Input Cell.
    PORT (
	ip	: IN BIT;	-- Input pad.
	ini	: OUT BIT;	-- Inverting output of cell.
	iz	: OUT BIT	-- Non-inverting output of cell.
    );
end PAincell;

architecture archPAincell of PAincell is
    begin
	ini	<= not(ip);
	iz	<= ip;

end archPAincell;
    
use work.mcpartspkg_local.all;
ENTITY PAbicell is		-- Master Bi-directional Cell.
    PORT (
	ie	: IN BIT;	-- Output Enable.
	i1	: IN BIT;	-- Inverting input.
	i2	: IN BIT;	-- Non-inverting input.
	iz	: OUT BIT;	-- Path into chip.
	ip	: INOUT x01z	-- I/O pad.
    );
end PAbicell;

architecture archPAbicell of PAbicell is
	signal ory	: bit;
    begin
	ory	<= i2 or not(i1);
	u1:bufoe port map (
		    oe	=> ie,
		    x	=> ory,
		    yfb	=> iz,
		    y	=> ip
		);
end archPAbicell;
	
use work.mcpartspkg_local.all;
ENTITY PAckcell is		-- Master Clock Cell.
    PORT (
	ip	: IN BIT;	-- Input pad.
	ini	: OUT BIT;	-- Inverted.
	iz	: OUT BIT;	-- Non-inverted.
	ic	: OUT BIT	-- To "Dedicated clock network".
    );
end PAckcell;

architecture archPAckcell of PAckcell is
    begin
	ini	<= not (ip);
	iz	<= ip;
	ic	<= ip;

end archPAckcell;

--	pASIC2 cells maped to this device.

use work.mcpartspkg_local.all;
ENTITY PAincel2 is		-- Master Input Cell.
    PORT (
	ip	: IN BIT;	-- Input pad.
	ini	: OUT BIT;	-- Inverting output of cell.
	iz	: OUT BIT;	-- Non-inverting output of cell.
	iqe	: IN BIT;	-- FF enable.
	iqc	: IN BIT;	-- FF clock.
	ir	: IN BIT;	-- FF reset.
	iq	: OUT BIT	-- FF output.
    );
end PAincel2;

architecture archPAincel2 of PAincel2 is
    signal ic : BIT;
begin

    ini	<= not(ip);
    iz	<= ip;
    ic <= iqe AND iqc;
    u1: dsrff port map (
	    s	=> zero,
	    clk	=> ic,
	    r	=> ir,
	    d	=> ip,
	    q	=> iq);

end archPAincel2;
    
use work.mcpartspkg_local.all;
ENTITY PAbicel2 is		-- Master Bi-directional Cell.
    PORT (
	ie	: IN BIT;	-- Output Enable.
	i1	: IN BIT;	-- Inverting input.
	i2	: IN BIT;	-- Non-inverting input.
	iz	: OUT BIT;	-- Path into chip.
	ip	: INOUT x01z;	-- I/O pad.
	iqe	: IN BIT;	-- FF enable.
	iqc	: IN BIT;	-- FF clock.
	ir	: IN BIT;	-- FF reset.
	iq	: OUT BIT	-- FF output.
    );
end PAbicel2;

architecture archPAbicel2 of PAbicel2 is
    signal ory,ic,d	: bit;
begin

    ory	<= i2 or not(i1);
    ic <= iqe AND iqc;
    u1:bufoe port map (
	    oe	=> ie,
	    x	=> ory,
	    yfb	=> d,
	    y	=> ip);
    iz <= d;
    u2: dsrff port map (
	    s	=> zero,
	    clk	=> ic,
	    r	=> ir,
	    d	=> d,
	    q	=> iq);

end archPAbicel2;
	
use work.mcpartspkg_local.all;
ENTITY PAckcel2 is		-- Master Clock Cell.
    PORT (
	ip	: IN BIT;	-- Input pad.
	ini	: OUT BIT;	-- Inverted.
	iz	: OUT BIT;	-- Non-inverted.
	ic	: OUT BIT;	-- To "Dedicated clock network".
	iqe	: IN BIT;	-- FF enable.
	iqc	: IN BIT;	-- FF clock.
	ir	: IN BIT;	-- FF reset.
	iq	: OUT BIT	-- FF output.
    );
end PAckcel2;

architecture archPAckcel2 of PAckcel2 is
    signal ick : BIT;
begin

    ini	<= not (ip);
    iz	<= ip;
    ic	<= ip;
    ick <= iqe AND iqc;
    u1: dsrff port map (
	    s	=> zero,
	    clk	=> ick,
	    r	=> ir,
	    d	=> ip,
	    q	=> iq);

end archPAckcel2;

use work.mcpartspkg_local.all;
entity clkbuf is
    PORT (
	a	: IN BIT;	-- Input.
	z	: OUT BIT	-- Output.
    );
end clkbuf;
architecture archclkbuf of clkbuf is
begin

    u1: buf port map (x => a, y => z);

end archclkbuf;

--		Fragment Cells:

use work.mcpartspkg_local.all;
ENTITY PAfrag_a is
    PORT (
	a1	: IN BIT;	-- Non-invert to AND A.
	a2	: IN BIT;	-- Invert to AND A.
	a3	: IN BIT;	-- Non-invert to AND A.
	a4	: IN BIT;	-- Invert to AND A.
	a5	: IN BIT;	-- Non-invert to AND A.
	a6	: IN BIT;	-- Invert to AND A.
	az	: OUT BIT	-- Output of AND A.
	);
end PAfrag_a;

architecture archPAfrag_a of PAfrag_a is
    begin
	az	<= a1 and not(a2) and a3 and not(a4) and a5 and not(a6);
end archPAfrag_a;

use work.mcpartspkg_local.all;
ENTITY PAfrag_f is		-- Fragment AND F.
    PORT (
	f1	: IN BIT;	-- Non-invert to AND F.
	f2	: IN BIT;	-- Invert to AND F.
	f3	: IN BIT;	-- Non-invert to AND F.
	f4	: IN BIT;	-- Invert to AND F.
	f5	: IN BIT;	-- Non-invert to AND F.
	f6	: IN BIT;	-- Invert to AND F.
	fz	: OUT BIT	-- Output of AND F.
	);
end PAfrag_f;

architecture archPAfrag_f of PAfrag_f is
    begin
	fz	<= f1 and not(f2) and f3 and not(f4) and f5 and not(f6);
end archPAfrag_f;
    
use work.mcpartspkg_local.all;
ENTITY PAfrag_m is		-- Fragment M.
    PORT (
	os	: IN BIT;	-- Select to MUX O.
	nsi	: IN BIT;	-- Select to MUX N.
	b1	: IN BIT;	-- Non-invert to AND B.
	b2	: IN BIT;	-- Invert to AND B.
	c1	: IN BIT;	-- Non-invert to AND C.
	c2	: IN BIT;	-- Invert to AND C.
	d1	: IN BIT;	-- Non-invert to AND D.
	d2	: IN BIT;	-- Invert to AND D.
	e1	: IN BIT;	-- Non-invert to AND E.
	e2	: IN BIT;	-- Invert to AND E.
	oz	: OUT BIT;	-- Output of MUX O.
	nz	: OUT BIT	-- Output of MUX N.
	);
end PAfrag_m;

architecture archPAfrag_m of PAfrag_m is
	signal by, cy, dy, ey, mzi, nzi, ozi : bit;
    begin
	by	<= b1 and not(b2);
	cy	<= c1 and not(c2);
	dy	<= d1 and not(d2);
	ey	<= e1 and not(e2);

	mzi	<= (cy and nsi) or (by and not(nsi));
	nzi	<= (ey and nsi) or (dy and not(nsi));
	nz	<= nzi;

	ozi	<= (nzi and os) or (mzi and not(os));
	oz	<= ozi;

end archPAfrag_m;

use work.mcpartspkg_local.all;
ENTITY PAfrag_q is		-- Fragment Q (flip flop).
    PORT (
	qs	: IN BIT;	-- Async Set on FF(Q).
	qc	: IN BIT;	-- Clock to FF(Q).
	qr	: IN BIT;	-- Async Reset on FF(Q).
	qd	: IN BIT;	-- Data on FF(Q).
	qz	: OUT BIT	-- Output of FF  Q.
	);
end PAfrag_q;

architecture archPAfrag_q of PAfrag_q is
begin
    u1	: dsrff
	port map (
	    s	=> qs,
	    clk	=> qc,
	    r	=> qr,
	    d	=> qd,
	    q	=> qz
	);
end archPAfrag_q;

use work.mcpartspkg_local.all;
ENTITY PAlcell is
    PORT (
	qs	: IN BIT;	-- Async Set on FF(Q).

	a1	: IN BIT;	-- Non-invert to AND A.
	a2	: IN BIT;	-- Invert to AND A.
	a3	: IN BIT;	-- Non-invert to AND A.
	a4	: IN BIT;	-- Invert to AND A.
	a5	: IN BIT;	-- Non-invert to AND A.
	a6	: IN BIT;	-- Invert to AND A.

	b1	: IN BIT;	-- Non-invert to AND B.
	b2	: IN BIT;	-- Invert to AND B.

	c1	: IN BIT;	-- Non-invert to AND C.
	c2	: IN BIT;	-- Invert to AND C.

	d1	: IN BIT;	-- Non-invert to AND D.
	d2	: IN BIT;	-- Invert to AND D.

	e1	: IN BIT;	-- Non-invert to AND E.
	e2	: IN BIT;	-- Invert to AND E.

	f1	: IN BIT;	-- Non-invert to AND F.
	f2	: IN BIT;	-- Invert to AND F.
	f3	: IN BIT;	-- Non-invert to AND F.
	f4	: IN BIT;	-- Invert to AND F.
	f5	: IN BIT;	-- Non-invert to AND F.
	f6	: IN BIT;	-- Invert to AND F.

	qc	: IN BIT;	-- Clock to FF(Q).
	qr	: IN BIT;	-- Async Reset on FF(Q).

	az	: OUT BIT;	-- Output of AND A.
	oz	: OUT BIT;	-- Output of MUX O.
	qz	: OUT BIT;	-- Output of FF  Q.
	nz	: OUT BIT;	-- Output of MUX N.
	fz	: OUT BIT	-- Output of AND F.
    );
END PAlcell;

architecture archPAlcell of PAlcell is
    signal aztmp,fztmp,oztmp : bit;
begin

	u1: pafrag_a port map (a1,a2,a3,a4,a5,a6,aztmp);
	u2: pafrag_m port map (aztmp,fztmp,b1,b2,c1,c2,d1,d2,e1,e2,oztmp,nz);
	u3: pafrag_f port map (f1,f2,f3,f4,f5,f6,fztmp);
	u4: pafrag_q port map (qs,qc,qr,oztmp,qz);

	az <= aztmp;
	fz <= fztmp;
	oz <= oztmp;

end archPAlcell;

use work.mcpartspkg_local.all;
ENTITY PAlcell2 is
    PORT (
	qs	: IN BIT;	-- Async Set on FF(Q).

	a1	: IN BIT;	-- Non-invert to AND A.
	a2	: IN BIT;	-- Invert to AND A.
	a3	: IN BIT;	-- Non-invert to AND A.
	a4	: IN BIT;	-- Invert to AND A.
	a5	: IN BIT;	-- Non-invert to AND A.
	a6	: IN BIT;	-- Invert to AND A.

	os	: IN BIT;       -- Data input MUX O.
	op	: IN BIT;       -- Select input MUX O.

	b1	: IN BIT;	-- Non-invert to AND B.
	b2	: IN BIT;	-- Invert to AND B.

	c1	: IN BIT;	-- Non-invert to AND C.
	c2	: IN BIT;	-- Invert to AND C.
	 
	mp	: IN BIT;       -- Select input MUX M.
	ms	: IN BIT;       -- Data input MUX M.
		      
	d1	: IN BIT;	-- Non-invert to AND D.
	d2	: IN BIT;	-- Invert to AND D.

	e1	: IN BIT;	-- Non-invert to AND E.
	e2	: IN BIT;	-- Invert to AND E.

	np	: IN BIT;       -- Select input MUX N.
	ns	: IN BIT;       -- Data input MUX N.
		     
	f1	: IN BIT;	-- Non-invert to AND F.
	f2	: IN BIT;	-- Invert to AND F.
	f3	: IN BIT;	-- Non-invert to AND F.
	f4	: IN BIT;	-- Invert to AND F.
	f5	: IN BIT;	-- Non-invert to AND F.
	f6	: IN BIT;	-- Invert to AND F.

	qc	: IN BIT;	-- Clock to FF(Q).
	qr	: IN BIT;	-- Async Reset on FF(Q).

	az	: OUT BIT;	-- Output of AND A.
	oz	: OUT BIT;	-- Output of MUX O.
	qz	: OUT BIT;	-- Output of FF  Q.
	nz	: OUT BIT;	-- Output of MUX N.
	fz	: OUT BIT	-- Output of AND F.
    );
END PAlcell2;

architecture archPAlcell2 of PAlcell2 is
    signal a,f,m,n,o,s0,s1,s2 : bit;
begin

	u1: pafrag_a port map (a1,a2,a3,a4,a5,a6,a);

	s0 <= (a AND op) OR (os AND NOT op);
	s1 <= (f AND mp) OR (ms AND NOT mp);
	s2 <= (f AND np) OR (ns AND NOT np);
	m <= (b1 AND NOT b2 AND NOT s1) OR (c1 AND NOT c2 AND s1);
	n <= (d1 AND NOT d2 AND NOT s2) OR (e1 AND NOT e2 AND s2);
	o <= (m AND NOT s0) OR (n AND s0);

	u3: pafrag_f port map (f1,f2,f3,f4,f5,f6,f);
	u4: pafrag_q port map (qs,qc,qr,o,qz);

	az <= a;
	fz <= f;
	oz <= o;
	nz <= n;

end archPAlcell2;

use work.mcpartspkg_local.all;
ENTITY PPAlcell is
    PORT (
	qs	: IN BIT;	-- Async Set on FF(Q).

	a1	: IN BIT;	-- Non-invert to AND A.
	a2	: IN BIT;	-- Invert to AND A.
	a3	: IN BIT;	-- Non-invert to AND A.
	a4	: IN BIT;	-- Invert to AND A.
	a5	: IN BIT;	-- Non-invert to AND A.
	a6	: IN BIT;	-- Invert to AND A.

	b1	: IN BIT;	-- Non-invert to AND B.
	b2	: IN BIT;	-- Invert to AND B.

	c1	: IN BIT;	-- Non-invert to AND C.
	c2	: IN BIT;	-- Invert to AND C.

	d1	: IN BIT;	-- Non-invert to AND D.
	d2	: IN BIT;	-- Invert to AND D.

	e1	: IN BIT;	-- Non-invert to AND E.
	e2	: IN BIT;	-- Invert to AND E.

	f1	: IN BIT;	-- Non-invert to AND F.
	f2	: IN BIT;	-- Invert to AND F.
	f3	: IN BIT;	-- Non-invert to AND F.
	f4	: IN BIT;	-- Invert to AND F.
	f5	: IN BIT;	-- Non-invert to AND F.
	f6	: IN BIT;	-- Invert to AND F.

	qc	: IN BIT;	-- Clock to FF(Q).
	qr	: IN BIT;	-- Async Reset on FF(Q).

	az	: OUT BIT;	-- Output of AND A.
	oz	: OUT BIT;	-- Output of MUX O.
	qz	: OUT BIT;	-- Output of FF  Q.
	nz	: OUT BIT;	-- Output of MUX N.
	fz	: OUT BIT	-- Output of AND F.
    );
END PPAlcell;

architecture archPPAlcell of PPAlcell is
begin

    u1: palcell port map (qs,a1,a2,a3,a4,a5,a6,b1,b2,c1,c2,d1,d2,e1,e2,
				f1,f2,f3,f4,f5,f6,qc,qr,az,oz,qz,nz,fz);

end archPPAlcell;

use work.mcpartspkg_local.all;
ENTITY PAlogic is
    PORT (
	qs	: IN BIT;	-- Async Set on FF(Q).

	a1	: IN BIT;	-- Non-invert to AND A.
	a2	: IN BIT;	-- Invert to AND A.
	a3	: IN BIT;	-- Non-invert to AND A.
	a4	: IN BIT;	-- Invert to AND A.
	a5	: IN BIT;	-- Non-invert to AND A.
	a6	: IN BIT;	-- Invert to AND A.

	b1	: IN BIT;	-- Non-invert to AND B.
	b2	: IN BIT;	-- Invert to AND B.

	c1	: IN BIT;	-- Non-invert to AND C.
	c2	: IN BIT;	-- Invert to AND C.

	d1	: IN BIT;	-- Non-invert to AND D.
	d2	: IN BIT;	-- Invert to AND D.

	e1	: IN BIT;	-- Non-invert to AND E.
	e2	: IN BIT;	-- Invert to AND E.

	f1	: IN BIT;	-- Non-invert to AND F.
	f2	: IN BIT;	-- Invert to AND F.
	f3	: IN BIT;	-- Non-invert to AND F.
	f4	: IN BIT;	-- Invert to AND F.
	f5	: IN BIT;	-- Non-invert to AND F.
	f6	: IN BIT;	-- Invert to AND F.

	qc	: IN BIT;	-- Clock to FF(Q).
	qr	: IN BIT;	-- Async Reset on FF(Q).

	az	: OUT BIT;	-- Output of AND A.
	oz	: OUT BIT;	-- Output of MUX O.
	qz	: OUT BIT;	-- Output of FF  Q.
	nz	: OUT BIT;	-- Output of MUX N.
	fz	: OUT BIT	-- Output of AND F.
    );
END PAlogic;

architecture archPAlogic of PAlogic is
begin

    u1: palcell port map (qs,a1,a2,a3,a4,a5,a6,b1,b2,c1,c2,d1,d2,e1,e2,
				f1,f2,f3,f4,f5,f6,qc,qr,az,oz,qz,nz,fz);

end archPAlogic;

use work.mcpartspkg_local.all;
ENTITY logico is
    PORT (
	a1	: IN BIT;	-- Non-invert to AND A.
	a2	: IN BIT;	-- Invert to AND A.
	a3	: IN BIT;	-- Non-invert to AND A.
	a4	: IN BIT;	-- Invert to AND A.
	a5	: IN BIT;	-- Non-invert to AND A.
	a6	: IN BIT;	-- Invert to AND A.

	b1	: IN BIT;	-- Non-invert to AND B.
	b2	: IN BIT;	-- Invert to AND B.

	c1	: IN BIT;	-- Non-invert to AND C.
	c2	: IN BIT;	-- Invert to AND C.

	d1	: IN BIT;	-- Non-invert to AND D.
	d2	: IN BIT;	-- Invert to AND D.

	e1	: IN BIT;	-- Non-invert to AND E.
	e2	: IN BIT;	-- Invert to AND E.

	f1	: IN BIT;	-- Non-invert to AND F.
	f2	: IN BIT;	-- Invert to AND F.
	f3	: IN BIT;	-- Non-invert to AND F.
	f4	: IN BIT;	-- Invert to AND F.
	f5	: IN BIT;	-- Non-invert to AND F.
	f6	: IN BIT;	-- Invert to AND F.

	oz	: OUT BIT	-- Output of MUX O.
    );
END logico;

architecture archlogico of logico is

	signal az,fz : bit;
begin

	u1: pafrag_a port map (a1,a2,a3,a4,a5,a6,az);
	u2: pafrag_m port map (az,fz,b1,b2,c1,c2,d1,d2,e1,e2,oz,open);
	u3: pafrag_f port map (f1,f2,f3,f4,f5,f6,fz);

end archlogico;
