--
--	Copyright Cypress Semiconductor Corporation, 1995,1996
--		as an unpublished work.
--

--
--	TTL82
--

ENTITY ttl82 IS
	PORT (C0 : IN BIT;
	      A1, A2, B1, B2: IN BIT;
              S1, S2: OUT BIT;
              C2: OUT BIT);
END ttl82;

ARCHITECTURE archttl82 OF ttl82 IS

    signal x1,x2,x3,x4 : Bit;
    attribute synthesis_off of x2,x4 :signal is true;

BEGIN

    x1 <= NOT ((NOT A1) AND (NOT B1));
    x2 <= NOT (A1 XOR B1);
    x3 <= NOT ((NOT A2) AND (NOT B2));
    x4 <= NOT (A2 XOR B2);
    S1 <= x2 XOR (NOT C0);
    S2 <= x4 XOR (NOT ((C0 AND x1) OR (x1 AND x2)));
    C2 <= (C0 AND x1 AND x3) OR (x1 AND x2 AND x3) OR (x3 AND x4);

END archttl82;

--
--	TTL83
--


ENTITY ttl83 IS
	PORT (C0 : IN BIT;
	      A1, A2, A3, A4, B1, B2, B3, B4 : IN BIT;
              S1, S2, S3, S4: OUT BIT;
              C4: OUT BIT);
END ttl83;

ARCHITECTURE archttl83 OF ttl83 IS

		        signal xp0,xp1,xg1,xp2,xg2,xp3,xg3 : Bit;
    attribute synthesis_off of xp0,xp1,xg1,xp2,xg2,xp3,xg3 : signal is true;
		        signal xc3,xc4 : Bit;
    attribute synthesis_off of xc3,xc4 : signal is true;
		        signal SUM1,SUM2,SUM3,SUM4,COUT : bit;
    attribute synthesis_off of SUM1,SUM2,SUM3,SUM4,COUT : signal is true;

BEGIN

    xp0 <= NOT ((NOT A1) AND (NOT B1));   -- propagate term for bit0
    xp1 <= NOT ((NOT A2) AND (NOT B2));   -- propagate term for bit1
    xg1 <= NOT (A2 AND B2);               -- generate term for bit1
    xp2 <= NOT ((NOT A3) AND (NOT B3));   -- propagate term for bit2
    xg2 <= NOT (A3 AND B3);               -- generate term for bit2
    xp3 <= NOT ((NOT A4) AND (NOT B4));   -- propagate term for bit3
    xg3 <= NOT (A4 AND B4);               -- generate term for bit3
    xc3 <= (A1 AND B1 AND xp1 AND xp2) OR (A2 AND B2 AND xp2) OR
           (A3 AND B3);
    xc4 <= ((A1 AND B1 AND xp1 AND xp2 AND xp3 AND xg3) OR
           (A2 AND B2 AND xp2 AND xp3 AND xg3) OR
           (A3 AND B3 AND xp3 AND xg3)) XOR (A4 AND B4);

    SUM1 <= NOT (((A1 AND B1) OR ((NOT A1) AND (NOT B1))) XOR C0);
    SUM2 <= (xp1 AND xg1) XOR ((A1 AND B1) OR (C0 AND xp0));
    SUM3 <= (xp2 AND xg2) XOR ((A1 AND B1 AND xp1) OR (C0 AND xp1 AND xp0)
            OR (A2 AND B2));
    SUM4 <= (xp3 AND xg3) XOR ((C0 AND xp2 AND xp1 AND xp0) OR xc3);
    COUT <= (C0 AND xp3 AND xp2 AND xp1 AND xp0) OR xc4;

    s1 <= SUM1;
    s2 <= SUM2;
    s3 <= SUM3;
    s4 <= SUM4;
    c4 <= COUT;

END archttl83;

--
--	TTL85
--

ENTITY ttl85 IS
        PORT (A0, A1, A2, A3, B0, B1, B2, B3: IN BIT;
 	      albi,aebi,agbi: IN BIT;
 	      alb,aeb,agb: OUT BIT);
END ttl85;

ARCHITECTURE archttl85 OF ttl85 IS

    signal x1aeqb,x2aeqb,x3aeqb,x4aeqb : bit;
    signal x21,x22,x23,x24,x_ab,xa_b : bit;
    attribute synthesis_off of x22,x24,xa_b : signal is true;
    attribute synthesis_off of x21,x23,x_ab : signal is true;

BEGIN

     x1aeqb <= NOT (A3 XOR B3);
     x2aeqb <= NOT (A2 XOR B2);
     x3aeqb <= NOT (A1 XOR B1);
     x4aeqb <= NOT (A0 XOR B0);
     xa_b <= NOT (A3 AND (NOT B3));
     x_ab <= NOT (B3 AND (NOT A3));
     x21 <= NOT (x1aeqb AND ((NOT A2) AND B2));
     x22 <= NOT (x1aeqb AND ((NOT B2) AND A2));
     x23 <= NOT (x1aeqb AND x2aeqb AND ((NOT A1) AND B1));
     x24 <= NOT (x1aeqb AND x2aeqb AND ((NOT B1) AND A1));
     AEB <= (x1aeqb AND x2aeqb AND x3aeqb AND x4aeqb AND AEBI);
     ALB <= ((AGBI AND x1aeqb AND x2aeqb AND x3aeqb AND x4aeqb AND x22 AND x24 AND xa_b)
          OR (AEBI AND x1aeqb AND x2aeqb AND x3aeqb AND x4aeqb AND x22 AND x24 AND xa_b)
          OR (A0 AND (NOT B0) AND x1aeqb AND x2aeqb AND x3aeqb AND x22 AND x24 AND xa_b))
           XOR (x22 AND x24 AND xa_b);
     AGB <= ((ALBI AND x1aeqb AND x2aeqb AND x3aeqb AND x4aeqb AND x21 AND x23 AND x_ab)
          OR (AEBI AND x1aeqb AND x2aeqb AND x3aeqb AND x4aeqb AND x21 AND x23 AND x_ab)
          OR ((NOT A0) AND B0 AND x1aeqb AND x2aeqb AND x3aeqb AND x21 AND x23 AND x_ab))
          XOR (x21 AND x23 AND x_ab);

END archttl85;

--
--	TTL148
--

ENTITY ttl148 IS
PORT (EI, I0, I1, I2, I3, I4, I5, I6, I7 : IN BIT;
	A0,A1,A2,GS,EO : OUT BIT);
END ttl148;

ARCHITECTURE archttl148 of ttl148 IS

    signal x1,x2 : Bit;
    attribute synthesis_off of x1 : signal is true;

BEGIN

   EO <= NOT (I0 AND I1 AND I2 AND I3 AND I4 AND I5 AND
	  I6 AND I7 AND (NOT EI));
   GS <= NOT ((NOT EI) AND NOT (I0 AND I1 AND I2 AND I3 AND
          I4 AND I5 AND I6 AND I7 AND (NOT EI)));
   x1 <= NOT ((NOT EI) AND (NOT I7));
   x2 <= ((NOT EI) AND x1 AND (NOT I1) AND I2 AND I4 AND I6) OR
         ((NOT EI) AND x1 AND (NOT I3) AND I4 AND I6) OR
         ((NOT EI) AND x1 AND (NOT I5) AND I6);
   A0 <= ((NOT x1) AND x2) OR (x1 AND (NOT x2));
   A1 <= NOT (((NOT I2) AND I4 AND I5 AND (NOT EI)) OR ((NOT I3)
          AND I4 AND I5 AND (NOT EI)) OR ((NOT I6) AND (NOT EI)) OR
          ((NOT I7) AND (NOT EI)));
   A2 <= NOT (((NOT I4) AND (NOT EI)) OR ((NOT I5) AND (NOT EI))
          OR ((NOT I6) AND (NOT EI)) OR ((NOT I7) AND (NOT EI)));

END archttl148;

--
--	TTL180
--

ENTITY ttl180 IS
	PORT (A,B,C,D,E,F,G,H, ODDI, EVEI: IN BIT;
	      ODD, EVE: OUT BIT);
END ttl180;

ARCHITECTURE archttl180 of ttl180 is

    signal x1,x2,x3,x4,x5,x6,x7,x8 : Bit;
    signal xodd,xeven : Bit;
    signal xabcd,xefgh : Bit;
    attribute synthesis_off of x1,x2,x3,x4 : signal is true;
    attribute synthesis_off of x5,x6,x7,x8 : signal is true;
    attribute synthesis_off of xodd,xeven : signal is true;
    attribute synthesis_off of xabcd,xefgh : signal is true;

BEGIN

    x1 <= NOT ((NOT B) AND C AND (NOT D));
    x2 <= NOT (B AND C AND D);
    x3 <= NOT ((NOT B) AND (NOT C) AND D);
    x4 <= NOT (B AND (NOT C) AND (NOT D));
    x5 <= NOT ((NOT f) AND G AND (NOT H));
    x6 <= NOT (F AND G AND H);
    x7 <= NOT ((NOT F) AND (NOT G) AND H);
    x8 <= NOT (F AND (NOT G) AND (NOT H));
    xabcd <= (x1 AND x2 AND x3 AND x4) XOR (NOT A);
    xefgh <= (x5 AND x6 AND x7 AND x8) XOR (NOT E);
    xodd  <= NOT ((NOT xabcd) AND (NOT xefgh) AND ODDI);
    xeven <= NOT ((NOT xabcd) AND (NOT xefgh) AND EVEI);
    EVE  <= ((xabcd AND xefgh AND ODDI AND xodd) OR
            ((NOT xabcd) AND xefgh AND EVEI AND xodd) OR
            (xabcd AND (NOT xefgh) AND EVEI AND xodd)) XOR xodd;
    ODD  <= ((xabcd AND xefgh AND EVEI AND xeven) OR
            ((NOT xabcd) AND xefgh AND ODDI AND xeven) OR
            (xabcd AND (NOT xefgh) AND ODDI AND xeven)) XOR xeven;

END archttl180;

--
--	TTL261
--

ENTITY ttl261 is
	PORT (C : IN BIT;
              B0,B1,B2,B3,B4 : IN BIT;
              M0,M1,M2 : IN BIT;
              Q0,Q1,Q2,Q3,Q4: OUT BIT);
END ttl261;

ARCHITECTURE archttl261 of ttl261 is

    signal                     a0,g0,c0,d0,e0,f0 : bit;
    attribute synthesis_off of a0,g0,c0,d0,e0,f0 : signal is true;
    signal                     a1,g1,c1,d1,e1,f1 : bit;
    attribute synthesis_off of a1,g1,c1,d1,e1,f1 : signal is true;
    signal                     a2,g2,c2,d2,e2,f2 : bit;
    attribute synthesis_off of a2,g2,c2,d2,e2,f2 : signal is true;
    signal                     a3,g3,c3,d3,e3,f3 : bit;
    attribute synthesis_off of a3,g3,c3,d3,e3,f3 : signal is true;
    signal                     a4,g4,c4,d4,e4,f4 : bit;
    attribute synthesis_off of a4,g4,c4,d4,e4,f4 : signal is true;
    signal m3,m4,m5,m6,m7: bit;

BEGIN

    m3 <= not ( (m0 and m1) or not (m0 or m1) );
    m4 <= ( m0 and m1 );
    m5 <= not ( m1 or m0 );
    m6 <= not ( m2 or not c );
    m7 <= ( m2 and c );

    a0 <= NOT (b0 and m4 and m6);
    g0 <= NOT (not b0 and m5 and m7);
    c0 <= NOT (b1 and m3 and m6);
    d0 <= NOT (not b1 and m3 and m7);
    e0 <= NOT (f0 and not c);
    f0 <= NOT (a0 and g0 and c0 and d0 and e0);
    q0 <= NOT (a0 and g0 and c0 and d0 and e0);

    a1 <= NOT (b1 and m4 and m6);
    g1 <= NOT (not b1 and m5 and m7);
    c1 <= NOT (b2 and m3 and m6);
    d1 <= NOT (not b2 and m3 and m7);
    e1 <= NOT (f1 and not c);
    f1 <= NOT (a1 and g1 and c1 and d1 and e1);
    q1 <= NOT (a1 and g1 and c1 and d1 and e1);

    a2 <= NOT (b2 and m4 and m6);
    g2 <= NOT (not b2 and m5 and m7);
    c2 <= NOT (b3 and m3 and m6);
    d2 <= NOT (not b3 and m3 and m7);
    e2 <= NOT (f2 and not c);
    f2 <= NOT (a2 and g2 and c2 and d2 and e2);
    q2 <= NOT (a2 and g2 and c2 and d2 and e2);

    a3 <= NOT (b3 and m4 and m6);
    g3 <= NOT (not b3 and m5 and m7);
    c3 <= NOT (b4 and m3 and m6);
    d3 <= NOT (not b4 and m3 and m7);
    e3 <= NOT (f3 and not c);
    f3 <= NOT (a3 and g3 and c3 and d3 and e3);
    q3 <= NOT (a3 and g3 and c3 and d3 and e3);

    a4 <= NOT (b4 and m4 and m6);
    g4 <= NOT (not b4 and m5 and m7);
    c4 <= NOT (b4 and m3 and m6);
    d4 <= NOT (not b4 and m3 and m7);
    e4 <= NOT (f4 and not c);
    f4 <= NOT (a4 and g4 and c4 and d4 and e4);
    q4 <= NOT f4;

END archttl261;

--
--	TTL518
--

ENTITY ttl518 IS
PORT (G, Q0, Q1, Q2, Q3, Q4, Q5, Q6, Q7,
	 P0, P1, P2, P3, P4, P5, P6, P7 : IN BIT;
      PQ : OUT BIT);
END ttl518;

ARCHITECTURE archttl518 of ttl518 is

    signal xpqn1,xpqn2,xpqn3,xpqn4,xpqn5,xpqn6,xpqn7 : bit;
    signal xpnq2,xpnq3,xpnq4,xpnq5,xpnq6,xpnq7 : bit;
    attribute synthesis_off of xpqn1,xpqn2,xpqn3,xpqn4,xpqn5,xpqn6,xpqn7 
							  : signal is true;
    attribute synthesis_off of xpnq2,xpnq3,xpnq4,xpnq5,xpnq6,xpnq7
							  : signal is true;

BEGIN

    xpqn1 <= NOT (P1 AND (NOT Q1));
    xpqn2 <= NOT (P2 AND (NOT Q2));
    xpqn3 <= NOT (P3 AND (NOT Q3));
    xpqn4 <= NOT (P4 AND (NOT Q4));
    xpqn5 <= NOT (P5 AND (NOT Q5));
    xpqn6 <= NOT (P6 AND (NOT Q6));
    xpqn7 <= NOT (P7 AND (NOT Q7));
    xpnq2 <= NOT ((NOT P2) AND Q2);
    xpnq3 <= NOT ((NOT P3) AND Q3);
    xpnq4 <= NOT ((NOT P4) AND Q4);
    xpnq5 <= NOT ((NOT P5) AND Q5);
    xpnq6 <= NOT ((NOT P6) AND Q6);
    xpnq7 <= NOT ((NOT P7) AND Q7);
    PQ <= ((xpqn1 AND xpqn2 AND xpqn3 AND xpqn4 AND xpqn5 AND xpqn6 AND
            xpqn7 AND xpnq2 AND xpnq3 AND xpnq4 AND xpnq5 AND xpnq6 AND
            xpnq7 AND (NOT P0) AND Q0 AND (NOT G)) OR
           (xpqn1 AND xpqn2 AND xpqn3 AND xpqn4 AND xpqn5 AND xpqn6 AND
            xpqn7 AND xpnq2 AND xpnq3 AND xpnq4 AND xpnq5 AND xpnq6 AND
            xpnq7 AND P0 AND (NOT Q0) AND (NOT G)) OR
           (xpqn1 AND xpqn2 AND xpqn3 AND xpqn4 AND xpqn5 AND xpqn6 AND
            xpqn7 AND xpnq2 AND xpnq3 AND xpnq4 AND xpnq5 AND xpnq6 AND
            xpnq7 AND (NOT P1) AND Q1 AND (NOT G))) XOR
           (xpqn1 AND xpqn2 AND xpqn3 AND xpqn4 AND xpqn5 AND xpqn6 AND
            xpqn7 AND xpnq2 AND xpnq3 AND xpnq4 AND xpnq5 AND xpnq6 AND
            xpnq7 AND (NOT G));

END archttl518;

