--
--      Copyright Cypress Semiconductor Corporation, 1995,1996
--	as an unpublished work.
--

--
--	MADD_SUB
--

library ieee;
use ieee.std_logic_1164.all;

package Madd_sub_local is
    type bvm		is array (integer range <>,integer range <>) of std_logic;
    type Shift_Type	is (LPM_LOGICAL, LPM_ROTATE, LPM_ARITHMETIC);
    type Repre_Type	is (LPM_SIGNED, LPM_UNSIGNED, LPM_GRAY);
    type Truth_Type	is (LPM_F, LPM_FD, LPM_FR, LPM_FDR);
    type CtDir_Type	is (LPM_NO_DIR, LPM_UP, LPM_DOWN);
    type Arith_Type	is (LPM_NO_TYP, LPM_ADD, LPM_SUB);
    type Fflop_Type	is (LPM_DFF,LPM_TFF);
    type ShDir_Type	is (LPM_LEFT,LPM_RIGHT);
    type goal_type	is (speed,area,combinatorial);
    type port_state_type is (ps_tied_low,ps_tied_high,ps_open,ps_connected);

    signal zero : std_logic := '0' ;
    signal one : std_logic := '1' ;

    function port_state (arg : std_logic) return port_state_type;
    function connects (x : std_logic; i : integer) return integer;
    function max (a,b : natural) return natural;
    function vectorize (a, size : natural) RETURN std_logic_vector;
    function restring (a : string; w : positive) return std_logic_vector;
component buf
    port(
	x	: in std_logic;		-- Input to Buffer.
	y	: out std_logic		-- Output from Buffer.
    );
end component;
end Madd_sub_local ;

package body Madd_sub_local is
    function port_state (arg : std_logic) return port_state_type is
	variable result : port_state_type := ps_connected;
    begin

	result := arg'port_state_att;

	return result;
    end port_state;

    function connects (x : std_logic; i : integer) return integer is
	variable result : integer := i;
    begin
	if (port_state(x) = ps_connected) then
	    result := result - 1;
	end if;
	return (result);
    end connects;

    function max (a,b : natural) return natural is
    begin
	if (a < b) then return b; else return a; end if;
    end max;

end Madd_sub_local ;

library ieee;
use ieee.std_logic_1164.all;

use work.Madd_sub_local.all ;

entity Madd_sub is
    generic(lpm_width	: positive;
	    lpm_representation	: repre_type;
	    lpm_direction	: arith_type;
	    lpm_hint	: goal_type);
    port(   dataA	: in	std_logic_vector((lpm_width-1) downto 0);
	    dataB	: in	std_logic_vector((lpm_width-1) downto 0);
	    cin		: in	std_logic;
	    add_sub	: in	std_logic;
	    result	: out	std_logic_vector((lpm_width-1) downto 0);
	    cout	: out	std_logic;
	    overflow	: out   std_logic);
end Madd_sub;

architecture archlpmstd of Madd_sub is

    signal c,ci,e,ei,r,ri,rtmp : std_logic_vector(result'length downto 0);
begin

    GA: if (((port_state(add_sub) = ps_tied_high)
	       AND (lpm_direction = lpm_no_typ))
	        OR (lpm_direction = lpm_add)) generate
	G0: if (lpm_width = 1) generate
	    rtmp(0) <= (dataA(0) XOR dataB(0)) XOR cin;
	    c(1) <= (dataA(0) AND dataB(0)) OR
		    (dataA(0) AND cin) OR
		    (dataB(0) AND cin);
	end generate;
	G1: if (lpm_width > 1) generate
	    rtmp(0) <= (dataA(0) XOR dataB(0)) XOR cin;
	    rtmp(1) <= (dataA(1) XOR dataB(1)) XOR
			((dataA(0) AND dataB(0)) OR (dataA(0) AND cin) OR
				      (dataB(0)  AND cin));
	    c(0) <=     ((dataA(0) AND dataB(0)  AND dataA(1)) OR
		         (dataA(0) AND dataB(0)  AND dataB(1)) OR
			      (cin AND dataB(0)  AND dataA(1)) OR
			      (cin AND dataB(0)  AND dataB(1)) OR
		         (dataA(0) AND   cin	 AND dataA(1)) OR
		         (dataA(0) AND   cin	 AND dataB(1)) OR
		         (dataA(1) AND		     dataB(1)));

	    UA0: buf port map (c(0), c(2));
	    ci(2) <= c(2);
	end generate;

	T1: if (result'length/2) /= 0 generate
	    GA0: for h in 1 to ((result'length/2)-1) generate
		rtmp(2*h) <=    ((dataA(2*h)   XOR dataB(2*h)) XOR c(2*h));
		rtmp(2*h+1) <=  ((dataA(2*h+1) XOR dataB(2*h+1)) XOR
				  ((dataA(2*h)   AND dataB(2*h)) OR
				   (dataA(2*h)   AND c(2*h)) OR
				   (dataB(2*h)   AND c(2*h))));
		GAS: if (lpm_hint = SPEED) generate
		    ei(2*h+2) <=  ((dataA(2*h+1) AND dataB(2*h+1)) OR
				  ((dataA(2*h+1) OR  dataB(2*h+1)) AND
				   (dataA(2*h)   AND dataB(2*h))));
		    ri(2*h+2) <=  ((dataA(2*h+1) OR  dataB(2*h+1)) AND
				   (dataA(2*h)   OR  dataB(2*h)));
		    ci(2*h+2) <= e(2*h+2) OR (ci(2*h) AND r(2*h+2));
		    U1: buf port map (ei(2*h+2),e(2*h+2));
		    U2: buf port map (ri(2*h+2),r(2*h+2));
		end generate;
		GAA: if (lpm_hint = AREA) generate
		    ci(2*h+2) <=((dataA(2*h) AND dataB(2*h) AND dataA(2*h+1)) OR
				 (dataA(2*h) AND dataB(2*h) AND dataB(2*h+1)) OR
				     (c(2*h) AND dataB(2*h) AND dataA(2*h+1)) OR
				     (c(2*h) AND dataB(2*h) AND dataB(2*h+1)) OR
				 (dataA(2*h) AND   c(2*h)   AND dataA(2*h+1)) OR
				 (dataA(2*h) AND   c(2*h)   AND dataB(2*h+1)) OR
				 (dataA(2*h+1)		    AND	dataB(2*h+1)));
		end generate;
		U3: buf port map (ci(2*h+2),c(2*h+2));
	    end generate;
	end generate;

	GA1: if (((result'length rem 2) /= 0) AND result'length /= 1) generate
	    rtmp(result'high)  <= ((dataA(result'high) XOR dataB(result'high))
					XOR c(result'high));
	    c(result'length) <= ((dataA(result'high) AND dataB(result'high)) OR
			         (dataA(result'high) AND c(result'high)) OR
			         (dataB(result'high) AND c(result'high)));
	end generate;
   end generate;

    GS: if (((port_state(add_sub) = ps_tied_low)
	       AND (lpm_direction = lpm_no_typ))
	        OR (lpm_direction = lpm_sub)) generate
	G0: if (lpm_width = 1) generate
	    rtmp(0) <= NOT(NOT dataA(0) XOR dataB(0)) XOR NOT cin;
	    c(1) <= (NOT dataA(0) AND dataB(0)) OR
		    (NOT dataA(0) AND NOT cin) OR
		    (dataB(0) AND NOT cin);
	end generate;
	G1: if (lpm_width > 1) generate
	    rtmp(0) <= (NOT(NOT(dataA(0) XOR dataB(0)) XOR NOT cin));
	    rtmp(1) <= (NOT(NOT(dataA(1) XOR dataB(1)) XOR
		     ((NOT dataA(0) AND dataB(0)) OR
		      (NOT dataA(0) AND NOT cin) OR (dataB(0) AND NOT cin))));
	    c(0) <=  ((NOT dataA(0) AND dataB(0)   AND NOT dataA(1)) OR
		      (NOT dataA(0) AND dataB(0)   AND     dataB(1)) OR
			   (NOT cin AND dataB(0)   AND NOT dataA(1)) OR
			   (NOT cin AND dataB(0)   AND     dataB(1)) OR
		      (NOT dataA(0) AND  NOT cin   AND NOT dataA(1)) OR
		      (NOT dataA(0) AND  NOT cin   AND     dataB(1)) OR
		      (NOT dataA(1) AND dataB(1)));
	    US0: buf port map (c(0), c(2));
	    ci(2) <= c(2);
	end generate;

	T1: if (result'length/2) /= 0 generate
	    GS0: for h in 1 to ((result'length/2)-1) generate
		rtmp(2*h)<= (NOT(NOT(dataA(2*h) XOR dataB(2*h)) XOR c(2*h)));
		rtmp(2*h+1) <= (NOT(NOT(dataA(2*h+1) XOR dataB(2*h+1)) XOR
				 ((NOT dataA(2*h)   AND dataB(2*h)) OR
				  (NOT dataA(2*h)   AND c(2*h)) OR
				      (dataB(2*h)   AND c(2*h)))));
		GSS: if (lpm_hint = SPEED) generate
		    ei(2*h+2) <= ((NOT dataA(2*h+1) AND dataB(2*h+1)) OR
				 ((NOT dataA(2*h+1) OR  dataB(2*h+1)) AND
				  (NOT dataA(2*h)   AND dataB(2*h))));
		    ri(2*h+2) <= ((NOT dataA(2*h+1) OR  dataB(2*h+1)) AND
				  (NOT dataA(2*h)   OR  dataB(2*h)));
		    ci(2*h+2) <= e(2*h+2) OR (ci(2*h) AND r(2*h+2));
		    U1: buf port map (ei(2*h+2),e(2*h+2));
		    U2: buf port map (ri(2*h+2),r(2*h+2));
		end generate;
		GSA: if (lpm_hint = AREA) generate
		ci(2*h+2)<=(NOT dataA(2*h) AND dataB(2*h) AND NOT dataA(2*h+1))
			OR (NOT dataA(2*h) AND dataB(2*h) AND     dataB(2*h+1))
			OR         (c(2*h) AND dataB(2*h) AND NOT dataA(2*h+1))
			OR         (c(2*h) AND dataB(2*h) AND     dataB(2*h+1))
			OR (NOT dataA(2*h) AND     c(2*h) AND NOT dataA(2*h+1))
			OR (NOT dataA(2*h) AND     c(2*h) AND     dataB(2*h+1))
			OR (NOT dataA(2*h+1)              AND     dataB(2*h+1));
		end generate;
		U3: buf port map (ci(2*h+2),c(2*h+2));
	    end generate;
	end generate;

	GS1: if (((result'length rem 2) /= 0) AND result'length /= 1) generate
	    rtmp(result'high) <= (NOT(NOT(dataA(result'high)
				   XOR dataB(result'high)) XOR c(result'high)));
	    c(result'length) <= ((NOT dataA(result'high) AND dataB(result'high))
				OR (NOT dataA(result'high) AND c(result'high))
				OR     (dataB(result'high) AND c(result'high)));
	end generate;
    end generate;

    GB: if ((port_state(add_sub) = ps_connected)
	      AND (lpm_direction = lpm_no_typ)) generate
	G0: if (lpm_width = 1) generate
	    rtmp(0) <=  (((dataA(0) XOR dataB(0)) XOR  cin) AND  add_sub) OR
		((NOT(NOT dataA(0) XOR dataB(0)) XOR NOT cin) AND NOT add_sub);
	    c(1) <= (((dataA(0) AND dataB(0)) OR
		    (dataA(0) AND cin) OR
		    (dataB(0) AND cin)) AND     add_sub) OR
	      (((NOT dataA(0) AND dataB(0)) OR
		    (NOT dataA(0) AND NOT cin) OR
		    (dataB(0) AND NOT cin)) AND NOT add_sub);
	end generate;
	G1: if (lpm_width > 1) generate
	    rtmp(0) <= (((dataA(0) XOR dataB(0)) XOR  cin) AND     add_sub) OR
		(NOT(NOT(dataA(0) XOR dataB(0)) XOR NOT cin) AND NOT add_sub);

	    rtmp(1) <= (((dataA(1) XOR dataB(1)) XOR
		       ((dataA(0) AND dataB(0)) OR  (    dataA(0) AND cin) OR
				     (dataB(0)  AND cin))) AND     add_sub) OR
		(NOT(NOT(dataA(1) XOR dataB(1)) XOR
		   ((NOT dataA(0) AND dataB(0)) OR (NOT dataA(0) AND NOT cin) OR
				     (dataB(0) AND NOT cin))) AND NOT add_sub);

	    c(0) <=   (((dataA(0) AND dataB(0)   AND     dataA(1)) OR
			(dataA(0) AND dataB(0)   AND     dataB(1)) OR
			     (cin AND dataB(0)   AND     dataA(1)) OR
			     (cin AND dataB(0)   AND     dataB(1)) OR
			(dataA(0) AND     cin	 AND     dataA(1)) OR
			(dataA(0) AND     cin	 AND     dataB(1)) OR
			(dataA(1) AND dataB(1))) AND      add_sub) OR
		  (((NOT dataA(0) AND dataB(0)   AND NOT dataA(1)) OR
		    (NOT dataA(0) AND dataB(0)   AND     dataB(1)) OR
			 (NOT cin AND dataB(0)   AND NOT dataA(1)) OR
			 (NOT cin AND dataB(0)   AND     dataB(1)) OR
		    (NOT dataA(0) AND NOT cin	 AND NOT dataA(1)) OR
		    (NOT dataA(0) AND NOT cin	 AND     dataB(1)) OR
		    (NOT dataA(1) AND dataB(1))) AND NOT  add_sub);

	UB0: buf port map (c(0), c(2));
	ci(2) <= c(2);
	end generate;

	T1: if (result'length/2) /= 0 generate
	    GB0: for h in 1 to ((result'length/2)-1) generate
		rtmp(2*h)<=((((dataA(2*h) XOR dataB(2*h)) XOR c(2*h)))
			AND     add_sub) OR
		    ((NOT(NOT(dataA(2*h) XOR dataB(2*h)) XOR c(2*h)))
			AND NOT add_sub);
		rtmp(2*h+1) <= (((dataA(2*h+1) XOR dataB(2*h+1)) XOR
			       ((dataA(2*h)   AND dataB(2*h)) OR
				(dataA(2*h)   AND c(2*h)) OR
				(dataB(2*h)   AND c(2*h)))) AND     add_sub) OR
			(NOT(NOT(dataA(2*h+1) XOR dataB(2*h+1)) XOR
			   ((NOT dataA(2*h)   AND dataB(2*h)) OR
			    (NOT dataA(2*h)   AND c(2*h)) OR
				(dataB(2*h)   AND c(2*h)))) AND NOT add_sub);

		GSS: if (lpm_hint = SPEED) generate
		  ei(2*h+2)<=(((dataA(2*h+1) AND dataB(2*h+1)) OR
			      ((dataA(2*h+1) OR  dataB(2*h+1)) AND
			       (dataA(2*h)   AND dataB(2*h)))) AND     add_sub)
		      OR (((NOT dataA(2*h+1) AND dataB(2*h+1)) OR
			  ((NOT dataA(2*h+1) OR  dataB(2*h+1)) AND
			   (NOT dataA(2*h)   AND dataB(2*h)))) AND NOT add_sub);
		  ri(2*h+2)<=(((dataA(2*h+1) OR  dataB(2*h+1)) AND
			       (dataA(2*h)   OR  dataB(2*h)))  AND     add_sub)
		      OR (((NOT dataA(2*h+1) OR  dataB(2*h+1)) AND
			   (NOT dataA(2*h)   OR  dataB(2*h)))  AND NOT add_sub);
		  ci(2*h+2) <= e(2*h+2) OR (ci(2*h) AND r(2*h+2));
		    U1: buf port map (ei(2*h+2),e(2*h+2));
		    U2: buf port map (ri(2*h+2),r(2*h+2));
		end generate;
		GAA: if (lpm_hint = AREA) generate
		  ci(2*h+2)<=(((dataA(2*h) AND dataB(2*h) AND     dataA(2*h+1))
			OR     (dataA(2*h) AND dataB(2*h) AND     dataB(2*h+1))
			OR         (c(2*h) AND dataB(2*h) AND     dataA(2*h+1))
			OR         (c(2*h) AND dataB(2*h) AND     dataB(2*h+1))
			OR     (dataA(2*h) AND     c(2*h) AND     dataA(2*h+1))
			OR     (dataA(2*h) AND     c(2*h) AND     dataB(2*h+1))
			OR     (dataA(2*h+1)              AND     dataB(2*h+1)))
						AND     add_sub) OR
			 (((NOT dataA(2*h) AND dataB(2*h) AND NOT dataA(2*h+1))
			OR (NOT dataA(2*h) AND dataB(2*h) AND     dataB(2*h+1))
			OR         (c(2*h) AND dataB(2*h) AND NOT dataA(2*h+1))
			OR         (c(2*h) AND dataB(2*h) AND     dataB(2*h+1))
			OR (NOT dataA(2*h) AND     c(2*h) AND NOT dataA(2*h+1))
			OR (NOT dataA(2*h) AND     c(2*h) AND     dataB(2*h+1))
			OR (NOT dataA(2*h+1)              AND     dataB(2*h+1)))
						AND NOT add_sub);
		end generate;
		U3: buf port map (ci(2*h+2),c(2*h+2));
	    end generate;
	end generate;

	GB1: if (((result'length rem 2) /= 0) AND result'length /= 1) generate
	    rtmp(result'high)  <=    ((((dataA(result'high)  XOR
					 dataB(result'high))  XOR
					     c(result'high))) AND     add_sub)
			    OR ((NOT(NOT(dataA(result'high)   XOR
					 dataB(result'high))  XOR
					     c(result'high))) AND NOT add_sub);

	    c(result'length) <= (((dataA(result'high) AND dataB(result'high)) OR
			   (dataA(result'high) AND c(result'high)) OR
			   (dataB(result'high) AND c(result'high))) AND add_sub)

		  OR (((NOT dataA(result'high) AND dataB(result'high)) OR
		       (NOT dataA(result'high) AND c(result'high)) OR
			   (dataB(result'high) AND c(result'high))) AND NOT add_sub);
	end generate;
    end generate;

    result <= rtmp(result'range);
    cout <= c(result'length) when (add_sub = '1') else NOT c(result'length);
    overflow <= ((rtmp(result'high) AND
	     NOT dataA(result'high) AND NOT dataB(result'high)) OR
	     (NOT rtmp(result'high) AND
		 dataA(result'high) AND     dataB(result'high)))
	 when (add_sub = '1') else
		((rtmp(result'high) AND
	     NOT dataA(result'high) AND     dataB(result'high)) OR
	     (NOT rtmp(result'high) AND
		 dataA(result'high) AND NOT dataB(result'high)));

end archlpmstd;
