--
--      Copyright Cypress Semiconductor Corporation, 1995,1996
--	as an unpublished work.
--

--
--	MLATCH
--

library ieee;
use ieee.std_logic_1164.all;

package Mlatch_local is
    type bvm		is array (integer range <>,integer range <>) of std_logic;
    type Shift_Type	is (LPM_LOGICAL, LPM_ROTATE, LPM_ARITHMETIC);
    type Repre_Type	is (LPM_SIGNED, LPM_UNSIGNED, LPM_GRAY);
    type Truth_Type	is (LPM_F, LPM_FD, LPM_FR, LPM_FDR);
    type CtDir_Type	is (LPM_NO_DIR, LPM_UP, LPM_DOWN);
    type Arith_Type	is (LPM_NO_TYP, LPM_ADD, LPM_SUB);
    type Fflop_Type	is (LPM_DFF,LPM_TFF);
    type ShDir_Type	is (LPM_LEFT,LPM_RIGHT);
    type goal_type	is (speed,area,combinatorial);
    type port_state_type is (ps_tied_low,ps_tied_high,ps_open,ps_connected);

    signal zero : std_logic := '0' ;
    signal one : std_logic := '1' ;

    function port_state (arg : std_logic) return port_state_type;
    function connects (x : std_logic; i : integer) return integer;
    function max (a,b : natural) return natural;
    function vectorize (a, size : natural) RETURN std_logic_vector;
    function restring (a : string; w : positive) return std_logic_vector;
component dsrlch 
    port (
	d	: in std_logic;
	s	: in std_logic;
	r	: in std_logic;
	e	: in std_logic;
	q	: out std_logic
    );
end component;
end Mlatch_local ;

package body Mlatch_local is
    function restring (a : string; w : positive) return std_logic_vector is
	variable result : std_logic_vector (w-1 downto 0) := (others => '1');
	variable indx : natural;
    begin

	if (a'length = 0) then
	    return result;
	elsif (a'length /= w) then
	    Assert (false)
	    Report "RESTRING: Source string length /= target vector width."
	    Severity Failure;
	    return result;
	else
	    indx := 0;
	    for i in a'reverse_range loop
		case a(i) is
		when '1' => result(indx) := '1';
		when '0' => result(indx) := '0';

		when '-' => result(indx) := '-';

		when others => Assert (false)

		    Report "Invalid char. literal (must be '1', '0', or '-').  Defauting to  '0'."

		    Severity Warning;
		    result(indx) := '0';
		end case;
		indx := indx + 1;
	    end loop;
	    return result;
	end if;
    end restring;
end Mlatch_local ;

library ieee;
use ieee.std_logic_1164.all;

use work.Mlatch_local.all;

entity Mlatch is
    generic(lpm_width	: positive;
	    lpm_avalue	: string;
	    lpm_pvalue	: string;
	    lpm_hint	: goal_type);
    port(   data	: in	std_logic_vector((lpm_width-1) downto 0);
	    gate	: in	std_logic;
	    q		: out	std_logic_vector((lpm_width-1) downto 0);
	    aset	: in	std_logic;
	    aclr	: in	std_logic);
end Mlatch;

architecture archlpmstd of Mlatch is

    signal qtmp,async,set,clr : std_logic_vector(q'high downto 0);

begin

    async <= restring(lpm_avalue,lpm_width);

    G0: for j in q'range generate
	set(j)    <= (aset AND     async(j));
	clr(j)    <= (aset AND NOT async(j)) OR aclr;
	UJ: dsrlch port map (data(j), set(j), clr(j), gate, qtmp(j));
    end generate;

    q <= qtmp;

end archlpmstd;
