--
--	Copyright Cypress Semiconductor Corporation, 1994,1995,1996
--	  as an unpublished work.
--
--------------------------------------------------------------------------
--		Components for Generic RTL.				--
--------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

PACKAGE rtlpkg IS

    COMPONENT bufoe		-- Tri-state input/output with feedback or I/O.
	PORT (
	    x	: IN std_logic;		-- Logic input to Buffer.
	    oe	: IN std_logic;		-- Output Enable input.
	    y	: inout std_logic;		-- std_logic output pin.
	    yfb	: OUT std_logic		-- Output feed back as std_logic.
	);
    END COMPONENT;
    attribute atomic_rtl of bufoe : component is bufoe_type;

    COMPONENT dltch		-- Transparent D latch.
	PORT (
	    d	: IN std_logic;		-- D input to latch.
	    e	: IN std_logic;		-- Enabel, tranparent high.
	    q	: OUT std_logic		-- Qutput from latch.
	);
    END COMPONENT;

    COMPONENT dff		-- D type flip flop. (only D and clk input)
	PORT (
	    d	: IN std_logic;		-- D input to register.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Output from register.
	);
    END COMPONENT;
    attribute atomic_rtl of dff : component is dff_type;

    COMPONENT xdff		-- X type flip flop. (X1, X2 and clk input)
	PORT (
	    x1	: IN std_logic;		-- First Exclusive OR input.
	    x2	: IN std_logic;		-- Second Exclusive OR input.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Qutput from register.
	);
    END COMPONENT;

    COMPONENT jkff		-- JK type flip flop. ( J, K and clk input)
	PORT (
	    j	: IN std_logic;		-- J input to J-K register.
	    k	: IN std_logic;		-- K input to J-K register.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Qutput from register.
	);
    END COMPONENT;

    COMPONENT buf		-- Buffer
	PORT (
	    x	: IN std_logic;		-- Input to Buffer.
	    y	: OUT std_logic		-- Output from Buffer.
	);
    END COMPONENT;
    attribute atomic_rtl of buf : component is buf_type;

    COMPONENT srlch		-- Set, Reset Latch. (S and R input only)
	PORT (
	    s	: IN std_logic;		-- Set input, Async.
	    r	: IN std_logic;		-- Reset input, Async.
	    q	: OUT std_logic		-- Output from Latch.
	);
    END COMPONENT;

    COMPONENT srff		-- Synchronous Set Reset Flip Flop.
	PORT (
	    s	: IN std_logic;		-- Set input, Sync.
	    r	: IN std_logic;		-- Reset inut, Sync.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Output from register.
	);
    END COMPONENT;

    COMPONENT dsrff		-- D type flip flop with async. set and reset.
	PORT (
	    d	: IN std_logic;		-- D input to register.
	    s	: IN std_logic;		-- Set input Async.
	    r	: IN std_logic;		-- Reset input Async.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Output from register.
	);
    END COMPONENT;
    attribute atomic_rtl of dsrff : component is dsrff_type;

    COMPONENT dprff		-- D type flip flop with async. set and reset.
	PORT (
	    d	: IN std_logic;		-- D input to register.
	    s	: IN std_logic;		-- Set input Async.
	    r	: IN std_logic;		-- Reset input Async.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Output from register.
	);
    END COMPONENT;
    attribute atomic_rtl of dprff : component is dprff_type;

    COMPONENT tsrff		-- T type flip flop with async. set and reset.
	PORT (
	    t	: IN std_logic;		-- T input to register.
	    s	: IN std_logic;		-- Set input Async.
	    r	: IN std_logic;		-- Reset input Async.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Output from register.
	);
    END COMPONENT;

    COMPONENT tff		-- T type flip flop. (T and clk input)
	PORT (
	    t	: IN std_logic;		-- Togal input to T type register.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Output from register.
	);
    END COMPONENT;

    COMPONENT xbuf		-- Xor primitive gate
	PORT (
	    x1	: IN std_logic;		-- Xor input.
	    x2	: IN std_logic;		-- Xor input.
	    q	: OUT std_logic		-- Xor output.
	);
    END COMPONENT;

    COMPONENT triout		-- Tri-state output with no feedback or I/O.
	PORT (
	    x	: IN std_logic;		-- Logic input to Buffer.
	    oe	: IN std_logic;		-- Output Enable input.
	    y	: OUT std_logic		-- std_logic output pin.
	);
    END COMPONENT;
    attribute atomic_rtl of triout : component is triout_type;

    -- C22VP10 specials

    -- need global reset, preset signals

    signal gar: std_logic;					-- Global Async Reset
    attribute node_num	of gar: signal is 25;		-- is node 25.

    signal gss: std_logic;					-- Global Sync Set
    attribute node_num	of gss: signal is 26;		-- is node 26.

    -- Nodal Special Macro Cell

    component MC_C22VP10
	generic (
	    is_inverted	: boolean := true	-- Select sense of output.
					    -- Does not affect sense of qfb_bar.
	);
	port(
	    d	: in std_logic;		-- D input to register.
	    oe	: in std_logic;		-- Output Enable.
	    clk	: in std_logic;		-- Positive edge clock.
	    y	: inout std_logic;		-- Output Pin.
	    qfb_bar	: out std_logic		-- Feedback from not(Q).
	);
    end component;
    attribute atomic_rtl of MC_C22VP10:component is MC_C22VP10_type;

    --  Schematic versions of macrocell.
    component MC22VP10I                      --  Inverted.
	port(
	    d	: in std_logic;		-- D input to register.
	    oe	: in std_logic;		-- Output Enable.
	    clk	: in std_logic;		-- Positive edge clock.
	    y	: inout std_logic;		-- Output Pin.
	    qfb_bar	: out std_logic		-- Feedback from not(Q).
	);
    end component;

    component MC22VP10N                      --  Not inverted.
	port(
	    d	: in std_logic;		-- D input to register.
	    oe	: in std_logic;		-- Output Enable.
	    clk	: in std_logic;		-- Positive edge clock.
	    y	: inout std_logic;		-- Output Pin.
	    qfb_bar	: out std_logic		-- Feedback from not(Q).
	);
    end component;

    END rtlpkg;

--------------------------------------------------------------------------
--		Enties for Generic RTL.					--
--------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY dltch IS
    PORT (
	d	: IN std_logic;		-- D input to latch.
	e	: IN std_logic;		-- Enabel, tranparent high.
	q	: OUT std_logic		-- Qutput from latch.
    );
END dltch;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY xdff IS
    PORT (
	x1	: IN std_logic;		-- First Exclusive OR input.
	x2	: IN std_logic;		-- Second Exclusive OR input.
	clk	: IN std_logic;		-- Positive edge Clock.
	q	: OUT std_logic		-- Qutput from register.
    );
END xdff;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY jkff IS
    PORT (
	j	: IN std_logic;		-- J input to J-K register.
	k	: IN std_logic;		-- K input to J-K register.
	clk	: IN std_logic;		-- Positive edge Clock.
	q	: OUT std_logic		-- Qutput from register.
    );
END jkff;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY srlch IS
    PORT (
	s	: IN std_logic;		-- Set input, Async.
	r	: IN std_logic;		-- Reset input, Async.
	q	: OUT std_logic		-- Output from Latch.
    );
END srlch;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY srff IS
    PORT (
	s	: IN std_logic;		-- Set input, Sync.
	r	: IN std_logic;		-- Reset inut, Sync.
	clk	: IN std_logic;		-- Positive edge Clock.
	q	: OUT std_logic		-- Output from register.
    );
END srff;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY tsrff IS	
    PORT (t	: IN std_logic;
	s	: IN std_logic;
	r	: IN std_logic;
	clk	: IN std_logic;
	q	: OUT std_logic);
END tsrff;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY tff IS
    PORT (
	t	: IN std_logic;		-- Togal input to T type register.
	clk	: IN std_logic;		-- Positive edge Clock.
	q	: OUT std_logic		-- Output from register.
    );
END tff;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY xbuf IS
    PORT (
	x1	: IN std_logic;		-- Xor input.
	x2	: IN std_logic;		-- Xor input.
	q	: OUT std_logic		-- Xor output.
    );
END xbuf;

--------------------------------------------------------------------------
--		Architectures for Generic RTL				--
--------------------------------------------------------------------------

ARCHITECTURE archdltch OF dltch IS
    SIGNAL qx : std_logic;
BEGIN
    qx	<= (d and e) or (d and qx) or (not(e) and qx);
    q	<= qx;
END archdltch;

ARCHITECTURE archxdff OF xdff IS
    SIGNAL dx : std_logic;
BEGIN
    dx	<= x1 xor x2;
    u1 :dff port map(dx,clk,q);
END archxdff;

ARCHITECTURE archjkff OF jkff IS
    SIGNAL qx, dx : std_logic;
BEGIN
    dx	<= (j and not(qx)) or (not(k) and qx);
    q	<= qx;
    d1:dff port map(dx,clk,qx);
END archjkff;
	
ARCHITECTURE archsrl OF srlch IS
    SIGNAL qx : std_logic;
BEGIN
    qx	<= (s and not(r)) or (qx and not(r));
    q	<= qx;
END archsrl;

ARCHITECTURE archsrff OF srff IS
    SIGNAL qx, dx : std_logic;
BEGIN
    dx  <= not ((not(s) or r) and (not (qx) or r));
    q	<= qx;
    d:dff port map(dx,clk,qx);
END archsrff;

ARCHITECTURE archTSRFF of tsrff IS
   signal dtmp,qtmp:std_logic;
BEGIN
	dtmp <= qtmp xor t;
	x:dsrff port map(dtmp,s,r,clk,qtmp);
	q <= qtmp;
END archTSRFF;

ARCHITECTURE archtff OF tff IS
    SIGNAL qx, dx : std_logic;
BEGIN
    dx	<= t xor qx;
    q	<= qx;
    d:dff port map(dx,clk,qx);
END archtff;

ARCHITECTURE archxbuf OF xbuf IS
BEGIN
    q	<= x1 xor x2;
END archxbuf;

library ieee;
use ieee.std_logic_1164.all;

use work.rtlpkg.all;
ENTITY xor2 IS
    port(
	a,b     : in std_logic;
	q       : out std_logic);
END xor2;
			 
ARCHITECTURE archXOR2 OF xor2 IS
begin
    u1 : xbuf port map (a, b, q);
END archXOR2;

--	Special Macro-cell definitions
--	Entities and  Architectrues

library ieee;
use ieee.std_logic_1164.all;

USE work.rtlpkg.all;
entity MC22VP10I is                      --  Inverted.
    port(
	d	: in std_logic;		-- D input to register.
	oe	: in std_logic;		-- Output Enable.
	clk	: in std_logic;		-- Positive edge clock.
	y	: inout std_logic;		-- Output Pin.
	qfb_bar	: out std_logic		-- Feedback from not(Q).
    );
end MC22VP10I;

ARCHITECTURE archMC22VP10I OF MC22VP10I IS

	signal q,qb : std_logic;

BEGIN
--  x1: MC_C22VP10 generic map (is_inverted => true)
--                port map (d, oe, clk, y, qfb_bar);

	U1: dff port map (d, clk, q);
	qb <= not q;
	qfb_bar <= not q;
	U2: bufoe port map (qb, oe, y, open);

END archMC22VP10I;

library ieee;
use ieee.std_logic_1164.all;

USE work.rtlpkg.all;
entity MC22VP10N is                     --  Not inverted.
    port(
	d	: in std_logic;		-- D input to register.
	oe	: in std_logic;		-- Output Enable.
	clk	: in std_logic;		-- Positive edge clock.
	y	: inout std_logic;		-- Output Pin.
	qfb_bar	: out std_logic		-- Feedback from not(Q).
    );
end MC22VP10N;

ARCHITECTURE archMC22VP10N OF MC22VP10N IS

	signal q : std_logic;

BEGIN
--  x1: MC_C22VP10 generic map (is_inverted => false)
--                port map (d, oe, clk, y, qfb_bar);

	U1: dff port map (d, clk, q);
	qfb_bar <= not q;
	U2: bufoe port map (q, oe, y, open);

END archMC22VP10N;

--      Here is the device entity.

library ieee;
use ieee.std_logic_1164.all;

entity C22VP10 IS
    PORT(
	pin1	: IN std_logic;
	pin2	: IN std_logic;
	pin3	: IN std_logic;
	pin4	: IN std_logic;
	pin5	: IN std_logic;
	pin6	: IN std_logic;
	pin7	: IN std_logic;
	pin8	: IN std_logic;
	pin9	: IN std_logic;
	pin10	: IN std_logic;
	pin11	: IN std_logic;
	pin13	: IN std_logic;
	pin14	: inout std_logic;
	pin15	: inout std_logic;
	pin16	: inout std_logic;
	pin17	: inout std_logic;
	pin18	: inout std_logic;
	pin19	: inout std_logic;
	pin20	: inout std_logic;
	pin21	: inout std_logic;
	pin22	: inout std_logic;
	pin23	: inout std_logic);
END C22VP10;
