--
--	Copyright Cypress Semiconductor Corporation, 1994,1995,1996
--	  as an unpublished work.
--
--------------------------------------------------------------------------
--		Components for Generic RTL.				--
--------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

PACKAGE rtlpkg IS

    COMPONENT bufoe		-- Tri-state input/output with feedback or I/O.
	PORT (
	    x	: IN std_logic;		-- Logic input to Buffer.
	    oe	: IN std_logic;		-- Output Enable input.
	    y	: inout std_logic;		-- std_logic output pin.
	    yfb	: OUT std_logic		-- Output feed back as std_logic.
	);
    END COMPONENT;
    attribute atomic_rtl of bufoe : component is bufoe_type;

    COMPONENT dltch		-- Transparent D latch.
	PORT (
	    d	: IN std_logic;		-- D input to latch.
	    e	: IN std_logic;		-- Enabel, tranparent high.
	    q	: OUT std_logic		-- Qutput from latch.
	);
    END COMPONENT;
    attribute atomic_rtl of dltch : component is dltch_type;

    COMPONENT dff		-- D type flip flop. (only D and clk input)
	PORT (
	    d	: IN std_logic;		-- D input to register.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Output from register.
	);
    END COMPONENT;
    attribute atomic_rtl of dff : component is dff_type;

    COMPONENT xdff		-- X type flip flop. (X1, X2 and clk input)
	PORT (
	    x1	: IN std_logic;		-- First Exclusive OR input.
	    x2	: IN std_logic;		-- Second Exclusive OR input.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Qutput from register.
	);
    END COMPONENT;
    attribute atomic_rtl of xdff : component is xdff_type;

    COMPONENT jkff		-- JK type flip flop. ( J, K and clk input)
	PORT (
	    j	: IN std_logic;		-- J input to J-K register.
	    k	: IN std_logic;		-- K input to J-K register.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Qutput from register.
	);
    END COMPONENT;
--    attribute atomic_rtl of jkff : component is jkff_type;
--	The previous line was removed for Release 1.1 ONLY.

    COMPONENT buf		-- Buffer
	PORT (
	    x	: IN std_logic;		-- Input to Buffer.
	    y	: OUT std_logic		-- Output from Buffer.
	);
    END COMPONENT;
    attribute atomic_rtl of buf : component is buf_type;

    COMPONENT srlch		-- Set, Reset Latch. (S and R input only)
	PORT (
	    s	: IN std_logic;		-- Set input, Async.
	    r	: IN std_logic;		-- Reset input, Async.
	    q	: OUT std_logic		-- Output from Latch.
	);
    END COMPONENT;

    COMPONENT srff		-- Synchronous Set Reset Flip Flop.
	PORT (
	    s	: IN std_logic;		-- Set input, Sync.
	    r	: IN std_logic;		-- Reset inut, Sync.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Output from register.
	);
    END COMPONENT;
--    attribute atomic_rtl of srff : component is srff_type;
--	The previous line was removed for Release 1.1 ONLY.

    COMPONENT dsrff		-- D type flip flop with async. set and reset.
	PORT (
	    d	: IN std_logic;		-- D input to register.
	    s	: IN std_logic;		-- Set input Async.
	    r	: IN std_logic;		-- Reset input Async.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Output from register.
	);
    END COMPONENT;
    attribute atomic_rtl of dsrff : component is dsrff_type;

    COMPONENT tsrff		-- T type flip flop with async. set and reset.
	PORT (
	    t	: IN std_logic;		-- T input to register.
	    s	: IN std_logic;		-- Set input Async.
	    r	: IN std_logic;		-- Reset input Async.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Output from register.
	);
    END COMPONENT;
    attribute atomic_rtl of tsrff : component is tsrff_type;

    COMPONENT tff		-- T type flip flop. (T and clk input)
	PORT (
	    t	: IN std_logic;		-- Togal input to T type register.
	    clk	: IN std_logic;		-- Positive edge Clock.
	    q	: OUT std_logic		-- Output from register.
	);
    END COMPONENT;
    attribute atomic_rtl of tff : component is tff_type;

    COMPONENT xbuf		-- Xor primitive gate
	PORT (
	    x1	: IN std_logic;		-- Xor input.
	    x2	: IN std_logic;		-- Xor input.
	    q	: OUT std_logic		-- Xor output.
	);
    END COMPONENT;
    attribute atomic_rtl of xbuf: component is xbuf_type;

    COMPONENT triout		-- Tri-state output with no feedback or I/O.
	PORT (
	    x	: IN std_logic;		-- Logic input to Buffer.
	    oe	: IN std_logic;		-- Output Enable input.
	    y	: OUT std_logic		-- std_logic output pin.
	);
    END COMPONENT;
    attribute atomic_rtl of triout : component is triout_type;

    type reg_sel	is (d_type,dlatch,comb); -- Register type selection.

    component MC_C340
	generic (
	    mcr	: reg_sel := d_type	-- Select type of register:
					-- Register, latch, or Comb.
	);
	port(
	    x1	: in std_logic;		-- Exor Macro Input (Single).
	    x2	: in std_logic;		-- Exor Macro Input (Multiple).
	    as	: in std_logic;		-- Async PreSet input.
	    ar	: in std_logic;		-- Async Reset input.
	    clk	: in std_logic;		-- Positive edge clock.
	    oe	: in std_logic;		-- Output Enable.
	    y	: out std_logic;		-- Macro Cell In/Out Pin.
	    fb	: out std_logic;		-- feedback from before buffer as std_logic.
	    yfb	: out std_logic		-- feedback from pin as std_logic.
	);
    end component;
    attribute atomic_rtl of MC_C340 : component is MC_C340_type;

    component MC340DL                   -- Latch
	port(
	    x1	: in std_logic;		-- Exor Macro Input (Single).
	    x2	: in std_logic;		-- Exor Macro Input (Multiple).
	    as	: in std_logic;		-- Async PreSet input.
	    ar	: in std_logic;		-- Async Reset input.
	    clk	: in std_logic;		-- Positive edge clock.
	    oe	: in std_logic;		-- Output Enable.
	    y	: out std_logic;		-- Macro Cell In/Out Pin.
	    fb	: out std_logic;		-- feedback from before buffer as std_logic.
	    yfb	: out std_logic		-- feedback from pin as std_logic.
	);
    end component;

    component MC340NR                   -- No register.
	port(
	    x1	: in std_logic;		-- Exor Macro Input (Single).
	    x2	: in std_logic;		-- Exor Macro Input (Multiple).
	    as	: in std_logic;		-- Async PreSet input.
	    ar	: in std_logic;		-- Async Reset input.
	    clk	: in std_logic;		-- Positive edge clock.
	    oe	: in std_logic;		-- Output Enable.
	    y	: out std_logic;		-- Macro Cell In/Out Pin.
	    fb	: out std_logic;		-- feedback from before buffer as std_logic.
	    yfb	: out std_logic		-- feedback from pin as std_logic.
	);
    end component;

    component MC340RG                   -- Register.
	port(
	    x1	: in std_logic;		-- Exor Macro Input (Single).
	    x2	: in std_logic;		-- Exor Macro Input (Multiple).
	    as	: in std_logic;		-- Async PreSet input.
	    ar	: in std_logic;		-- Async Reset input.
	    clk	: in std_logic;		-- Positive edge clock.
	    oe	: in std_logic;		-- Output Enable.
	    y	: out std_logic;		-- Macro Cell In/Out Pin.
	    fb	: out std_logic;		-- feedback from before buffer as std_logic.
	    yfb	: out std_logic		-- feedback from pin as std_logic.
	);
    end component;

    component MC_C340_b
	generic (
	    mcr	: reg_sel := d_type	-- Select type of register:
				        -- Register, latch, or Comb.
	);
	port(
	    x1	: in std_logic;		-- Exor Macro Input (Single).
	    x2	: in std_logic;		-- Exor Macro Input (Multiple).
	    as	: in std_logic;		-- Async PreSet input.
	    ar	: in std_logic;		-- Async Reset input.
	    clk	: in std_logic;		-- Positive edge clock.
	    fb	: out std_logic		-- Macro Cell Out std_logic.
	);
    end component;
    attribute atomic_rtl of MC_C340_b : component is MC_C340_b_type;

    component MC340BDL                  -- Latch
	port(
	x1	: in std_logic;		-- Exor Macro Input (Single).
	x2	: in std_logic;		-- Exor Macro Input (Multiple).
	as	: in std_logic;		-- Async PreSet input.
	ar	: in std_logic;		-- Async Reset input.
	clk	: in std_logic;		-- Positive edge clock.
	fb	: out std_logic		-- Macro Cell Out std_logic.
	);
    end component;

    component MC340BNR                  -- No register.
	port(
	    x1	: in std_logic;		-- Exor Macro Input (Single).
	    x2	: in std_logic;		-- Exor Macro Input (Multiple).
	    as	: in std_logic;		-- Async PreSet input.
	    ar	: in std_logic;		-- Async Reset input.
	    clk	: in std_logic;		-- Positive edge clock.
	    fb	: out std_logic		-- Macro Cell Out std_logic.
	);
    end component;

    component MC340BRG                  -- Register.
	port(
	    x1	: in std_logic;		-- Exor Macro Input (Single).
	    x2	: in std_logic;		-- Exor Macro Input (Multiple).
	    as	: in std_logic;		-- Async PreSet input.
	    ar	: in std_logic;		-- Async Reset input.
	    clk	: in std_logic;		-- Positive edge clock.
	    fb	: out std_logic		-- Macro Cell Out std_logic.
	);
    end component;

    component fb_C340
	port(
	    exp	: in std_logic;		-- Input to expander.
	    fb	: out std_logic		-- Output from expander.
	);
    end component;
    attribute atomic_rtl of fb_C340 : component is fb_C340_type;

END rtlpkg;

--------------------------------------------------------------------------
--		Enties and Archetectures for Generic RTL.		--
--------------------------------------------------------------------------

--
-- FOR Release 1.1 ONLY.

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY srff IS
    PORT (
	s	: IN std_logic;		-- First Exclusive OR input.
	r	: IN std_logic;		-- Second Exclusive OR input.
	clk	: IN std_logic;		-- Positive edge Clock.
	q	: OUT std_logic		-- Qutput from register.
    );
END srff;

ARCHITECTURE archsrff OF srff IS
    SIGNAL dx,qx : std_logic;
BEGIN
    dx	<= (s AND NOT r) OR (NOT r AND qx);
    u1 :dff port map(dx,clk,qx);
    q <= qx;
END archsrff;

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY jkff IS
    PORT (
	j	: IN std_logic;		-- J input to J-K register.
	k	: IN std_logic;		-- K input to J-K register.
	clk	: IN std_logic;		-- Positive edge Clock.
	q	: OUT std_logic		-- Qutput from register.
    );
END jkff;

ARCHITECTURE archjkff OF jkff IS
    SIGNAL qx, dx : std_logic;
BEGIN
    dx	<= (j and not(qx)) or (not(k) and qx);
    q	<= qx;
    d1:dff port map(dx,clk,qx);
END archjkff;
-- FOR Release 1.1 ONLY.
--

library ieee;
use ieee.std_logic_1164.all;

USE WORK.rtlpkg.ALL;
ENTITY srlch IS
    PORT (
	s       : IN std_logic;               -- Set input, Async.
	r       : IN std_logic;               -- Reset input, Async.
	q       : OUT std_logic               -- Output from Latch.
    );
END srlch;

ARCHITECTURE archsrl OF srlch IS
    SIGNAL qx : std_logic;
BEGIN
    qx  <= (s and not(r)) or (qx and not(r));
    q   <= qx;
END archsrl;

-- XOR2 is a special case handled on a by device basis because some
-- devices have a two input XOR as part of their atomic structure.

library ieee;
use ieee.std_logic_1164.all;

use work.rtlpkg.all;
ENTITY xor2 IS
    port(
	a,b     : in std_logic;
	q       : out std_logic);
END xor2;
			 
ARCHITECTURE archXOR2 OF xor2 IS
begin
    u1 : xbuf port map (a, b, q);
END archXOR2;

--  Schematic versions

library ieee;
use ieee.std_logic_1164.all;

USE work.rtlpkg.all;
entity MC340DL is                       -- Latch
    port(
	x1	: in std_logic;		-- Exor Macro Input (Single).
	x2	: in std_logic;		-- Exor Macro Input (Multiple).
	as	: in std_logic;		-- Async PreSet input.
	ar	: in std_logic;		-- Async Reset input.
	clk	: in std_logic;		-- Positive edge clock.
	oe	: in std_logic;		-- Output Enable.
	y	: out std_logic;		-- Macro Cell In/Out Pin.
	fb	: out std_logic;		-- feedback from before buffer as std_logic.
	yfb	: out std_logic		-- feedback from pin as std_logic.
    );
end MC340DL;

ARCHITECTURE struct OF MC340DL IS

BEGIN  --  struct 
x1:MC_C340 generic map(dlatch)
          port map(x1, x2, as, ar, clk, oe, y, fb, yfb);
END struct;

library ieee;
use ieee.std_logic_1164.all;

USE work.rtlpkg.all;
entity MC340NR is                       -- No register.
    port(
	x1	: in std_logic;		-- Exor Macro Input (Single).
	x2	: in std_logic;		-- Exor Macro Input (Multiple).
	as	: in std_logic;		-- Async PreSet input.
	ar	: in std_logic;		-- Async Reset input.
	clk	: in std_logic;		-- Positive edge clock.
	oe	: in std_logic;		-- Output Enable.
	y	: out std_logic;		-- Macro Cell In/Out Pin.
	fb	: out std_logic;		-- feedback from before buffer as std_logic.
	yfb	: out std_logic		-- feedback from pin as std_logic.
    );
end MC340NR;

ARCHITECTURE struct OF MC340NR IS

BEGIN  --  struct 
x1:MC_C340 generic map(comb)
          port map(x1, x2, as, ar, clk, oe, y, fb, yfb);
END struct;

library ieee;
use ieee.std_logic_1164.all;

USE work.rtlpkg.all;
entity MC340RG is                       -- Register.
    port(
	x1	: in std_logic;		-- Exor Macro Input (Single).
	x2	: in std_logic;		-- Exor Macro Input (Multiple).
	as	: in std_logic;		-- Async PreSet input.
	ar	: in std_logic;		-- Async Reset input.
	clk	: in std_logic;		-- Positive edge clock.
	oe	: in std_logic;		-- Output Enable.
	y	: out std_logic;		-- Macro Cell In/Out Pin.
	fb	: out std_logic;		-- feedback from before buffer as std_logic.
	yfb	: out std_logic		-- feedback from pin as std_logic.
    );
end MC340RG;

ARCHITECTURE struct OF MC340RG IS

BEGIN  --  struct 
x1:MC_C340 generic map(d_type)
          port map(x1, x2, as, ar, clk, oe, y, fb, yfb);
END struct;

library ieee;
use ieee.std_logic_1164.all;

USE work.rtlpkg.all;
entity MC340BDL is                      -- Latch
    port(
	x1	: in std_logic;		-- Exor Macro Input (Single).
	x2	: in std_logic;		-- Exor Macro Input (Multiple).
	as	: in std_logic;		-- Async PreSet input.
	ar	: in std_logic;		-- Async Reset input.
	clk	: in std_logic;		-- Positive edge clock.
	fb	: out std_logic		-- Macro Cell Out std_logic.
    );
end MC340BDL;

ARCHITECTURE struct OF MC340BDL IS

BEGIN  --  struct 
x1:MC_C340_B generic map(dlatch)
          port map(x1, x2, as, ar, clk, fb);
END struct;

library ieee;
use ieee.std_logic_1164.all;

USE work.rtlpkg.all;
entity MC340BNR is                      -- No register.
    port(
	x1	: in std_logic;		-- Exor Macro Input (Single).
	x2	: in std_logic;		-- Exor Macro Input (Multiple).
	as	: in std_logic;		-- Async PreSet input.
	ar	: in std_logic;		-- Async Reset input.
	clk	: in std_logic;		-- Positive edge clock.
	fb	: out std_logic		-- Macro Cell Out std_logic.
    );
end MC340BNR;

ARCHITECTURE struct OF MC340BNR IS

BEGIN  --  struct 
x1:MC_C340_B generic map(comb)
          port map(x1, x2, as, ar, clk, fb);
END struct;

library ieee;
use ieee.std_logic_1164.all;

USE work.rtlpkg.all;
entity MC340BRG is                      -- Register.
    port(
	x1	: in std_logic;		-- Exor Macro Input (Single).
	x2	: in std_logic;		-- Exor Macro Input (Multiple).
	as	: in std_logic;		-- Async PreSet input.
	ar	: in std_logic;		-- Async Reset input.
	clk	: in std_logic;		-- Positive edge clock.
	fb	: out std_logic		-- Macro Cell Out std_logic.
    );
end MC340BRG;

ARCHITECTURE struct OF MC340BRG IS

BEGIN  --  struct 
x1:MC_C340_B generic map(d_type)
          port map(x1, x2, as, ar, clk, fb);
END struct;

------------------------------------------------------------------
--		Entities for all max parts:			--
--		  341, 342, 343, 344, 344a, 344s, 346.		--
------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity c341 is
    port (
	pin1	: in std_logic;
	pin2	: in std_logic;
	pin4	: inout std_logic;
	pin5	: inout std_logic;
	pin6	: inout std_logic;
	pin7	: inout std_logic;
	pin8	: inout std_logic;
	pin9	: inout std_logic;
	pin10	: inout std_logic;
	pin11	: inout std_logic;
	pin12	: inout std_logic;
	pin13	: inout std_logic;
	pin14	: inout std_logic;
	pin15	: inout std_logic;
	pin16	: inout std_logic;    
	pin17	: inout std_logic;
	pin20	: inout std_logic;
	pin21	: inout std_logic;
	pin22	: inout std_logic;
	pin23	: inout std_logic;
	pin25	: inout std_logic;
	pin26	: inout std_logic;
	pin27	: inout std_logic;
	pin28	: inout std_logic;
	pin29	: inout std_logic;
	pin30	: inout std_logic;
	pin31	: inout std_logic;  
	pin32	: inout std_logic;
	pin33	: inout std_logic;    
	pin34	: inout std_logic;
	pin35	: inout std_logic;
	pin36	: inout std_logic;    
	pin37	: inout std_logic;
	pin38	: inout std_logic;
	pin41	: in std_logic;
	pin42	: in std_logic;
	pin43	: in std_logic;
	pin44	: in std_logic;
	pin46	: inout std_logic;
	pin47	: inout std_logic;
	pin48	: inout std_logic;
	pin49	: inout std_logic;
	pin50	: inout std_logic;    
	pin51	: inout std_logic;
	pin52	: inout std_logic;
	pin53	: inout std_logic;
	pin54	: inout std_logic;
	pin55	: inout std_logic;
	pin56	: inout std_logic;
	pin57	: inout std_logic;
	pin58	: inout std_logic;
	pin59	: inout std_logic;  
	pin62	: inout std_logic;
	pin63	: inout std_logic;
	pin64	: inout std_logic;
	pin65	: inout std_logic;
	pin67	: inout std_logic;    
	pin68	: inout std_logic;
	pin69	: inout std_logic;
	pin70	: inout std_logic;
	pin71	: inout std_logic;
	pin72	: inout std_logic;
	pin73	: inout std_logic;
	pin74	: inout std_logic;
	pin75	: inout std_logic;
	pin76	: inout std_logic;
	pin77	: inout std_logic;
	pin78	: inout std_logic;
	pin79	: inout std_logic;
	pin80	: inout std_logic;
	pin83	: in std_logic;
	pin84	: in std_logic);
end C341;

library ieee;
use ieee.std_logic_1164.all;

entity c342 is
    port (
	pin1	: in std_logic;
	pin2	: in std_logic;  
	pin4	: inout std_logic;
	pin5	: inout std_logic;
	pin6	: inout std_logic;
	pin7	: inout std_logic;
	pin8	: inout std_logic;
	pin9	: inout std_logic;
	pin10	: inout std_logic;
	pin11	: inout std_logic;
	pin12	: inout std_logic;
	pin13	: inout std_logic;
	pin14	: inout std_logic;
	pin15	: inout std_logic;
	pin17	: inout std_logic;
	pin18	: inout std_logic;
	pin19	: inout std_logic;
	pin21	: inout std_logic;
	pin22	: inout std_logic;
	pin23	: inout std_logic;
	pin24	: inout std_logic;
	pin25	: inout std_logic;
	pin26	: inout std_logic;
	pin27	: inout std_logic;
	pin28	: inout std_logic;
	pin29	: inout std_logic;
	pin30	: inout std_logic;
	pin31	: inout std_logic;  
	pin32	: in std_logic;
	pin34	: in std_logic;
	pin35	: in std_logic;
	pin36	: in std_logic;    
	pin38	: inout std_logic;
	pin39	: inout std_logic;
	pin40	: inout std_logic;
	pin41	: inout std_logic;
	pin42	: inout std_logic;
	pin43	: inout std_logic;
	pin44	: inout std_logic;
	pin45	: inout std_logic;
	pin46	: inout std_logic;
	pin47	: inout std_logic;
	pin48	: inout std_logic;
	pin49	: inout std_logic;
	pin51	: inout std_logic;
	pin52	: inout std_logic;
	pin53	: inout std_logic;
	pin55	: inout std_logic;
	pin56	: inout std_logic;
	pin57	: inout std_logic;
	pin58	: inout std_logic;
	pin59	: inout std_logic;  
	pin60	: inout std_logic;
	pin61	: inout std_logic;
	pin62	: inout std_logic;
	pin63	: inout std_logic;
	pin64	: inout std_logic;
	pin65	: inout std_logic;
	pin66	: in std_logic;
	pin68	: in std_logic);
end c342;

library ieee;
use ieee.std_logic_1164.all;

entity c343 is
    port (
	pin1	: inout std_logic; 
	pin2	: inout std_logic;  
	pin4	: inout std_logic;
	pin5	: inout std_logic;
	pin6	: inout std_logic;
	pin7	: inout std_logic;
	pin8	: inout std_logic;
	pin9	: in std_logic;
	pin11	: in std_logic;
	pin12	: in std_logic;
	pin13	: in std_logic;
	pin15	: inout std_logic;
	pin16	: inout std_logic;    
	pin17	: inout std_logic;
	pin18	: inout std_logic;
	pin19	: inout std_logic;
	pin20	: inout std_logic;
	pin22	: inout std_logic;
	pin23	: inout std_logic;
	pin24	: inout std_logic;
	pin26	: inout std_logic;
	pin27	: inout std_logic;
	pin28	: inout std_logic;
	pin29	: inout std_logic;
	pin30	: inout std_logic;
	pin31	: in std_logic;  
	pin33	: in std_logic;    
	pin34	: in std_logic; 
	pin35	: in std_logic;
	pin37	: inout std_logic;
	pin38	: inout std_logic;
	pin39	: inout std_logic;
	pin40	: inout std_logic;
	pin41	: inout std_logic;
	pin42	: inout std_logic;
	pin44	: inout std_logic);
end C343;

library ieee;
use ieee.std_logic_1164.all;

entity C344 is
    port (
	pin1	: in std_logic;   
	pin2	: in std_logic;
	pin3	: inout std_logic;  
	pin4	: inout std_logic;
	pin5	: inout std_logic;
	pin6	: inout std_logic;
	pin9	: inout std_logic;
	pin10	: inout std_logic;
	pin11	: inout std_logic;
	pin12	: inout std_logic;
	pin13	: in std_logic;
	pin14	: in std_logic;
	pin15	: in std_logic;  
	pin16	: in std_logic;
	pin17	: inout std_logic;
	pin18	: inout std_logic;
	pin19	: inout std_logic;
	pin20	: inout std_logic;
	pin23	: inout std_logic;
	pin24	: inout std_logic;
	pin25	: inout std_logic;
	pin26	: inout std_logic;
	pin27	: in std_logic;
	pin28	: in std_logic);
end C344;

library ieee;
use ieee.std_logic_1164.all;

entity c344a is
    port (
	pin1	: in std_logic;   
	pin2	: in std_logic;
	pin3	: inout std_logic;  
	pin4	: inout std_logic;
	pin5	: inout std_logic;
	pin6	: inout std_logic;
	pin9	: inout std_logic;
	pin10	: inout std_logic;
	pin11	: inout std_logic;
	pin12	: inout std_logic;
	pin13	: in std_logic;
	pin14	: in std_logic;
	pin15	: in std_logic;  
	pin16	: in std_logic;
	pin17	: inout std_logic;
	pin18	: inout std_logic;
	pin19	: inout std_logic;
	pin20	: inout std_logic;
	pin23	: inout std_logic;
	pin24	: inout std_logic;
	pin25	: inout std_logic;
	pin26	: inout std_logic;
	pin27	: in std_logic;
	pin28	: in std_logic
    );
end c344a;

library ieee;
use ieee.std_logic_1164.all;

entity c344s is
    port (
	pin1	: in std_logic;   
	pin2	: in std_logic;
	pin3	: inout std_logic;  
	pin4	: inout std_logic;
	pin5	: inout std_logic;
	pin6	: inout std_logic;
	pin9	: inout std_logic;
	pin10	: inout std_logic;
	pin11	: inout std_logic;
	pin12	: inout std_logic;
	pin13	: in std_logic;
	pin14	: in std_logic;
	pin15	: in std_logic;  
	pin16	: in std_logic;
	pin17	: inout std_logic;
	pin18	: inout std_logic;
	pin19	: inout std_logic;
	pin20	: inout std_logic;
	pin23	: inout std_logic;
	pin24	: inout std_logic;
	pin25	: inout std_logic;
	pin26	: inout std_logic;
	pin27	: in std_logic;
	pin28	: in std_logic
    );
end c344s;

library ieee;
use ieee.std_logic_1164.all;

entity c346 is
    port (
	pin1	: in std_logic;
	pin2	: in std_logic;
	pin5	: in std_logic;
	pin6	: in std_logic;
	pin7	: in std_logic;
	pin8	: inout std_logic;
	pin9	: inout std_logic;
	pin10	: inout std_logic;
	pin11	: inout std_logic;
	pin12	: inout std_logic;
	pin13	: inout std_logic;
	pin14	: inout std_logic;
	pin15	: inout std_logic;
	pin16	: inout std_logic;    
	pin17	: inout std_logic;
	pin18	: inout std_logic;
	pin21	: inout std_logic;
	pin22	: inout std_logic;
	pin25	: inout std_logic;
	pin26	: inout std_logic;
	pin27	: inout std_logic;
	pin28	: inout std_logic;
	pin29	: inout std_logic;
	pin30	: inout std_logic;
	pin31	: inout std_logic;  
	pin32	: inout std_logic;
	pin33	: inout std_logic;    
	pin34	: inout std_logic;
	pin35	: inout std_logic;
	pin36	: in std_logic;
	pin37	: in std_logic;
	pin38	: in std_logic;
	pin41	: in std_logic;
	pin42	: in std_logic;
	pin43	: in std_logic;
	pin44	: in std_logic;
	pin47	: in std_logic;
	pin48	: in std_logic;
	pin49	: in std_logic;
	pin50	: inout std_logic;    
	pin51	: inout std_logic;
	pin52	: inout std_logic;
	pin53	: inout std_logic;
	pin54	: inout std_logic;
	pin55	: inout std_logic;
	pin56	: inout std_logic;
	pin57	: inout std_logic;
	pin58	: inout std_logic;
	pin59	: inout std_logic;  
	pin60	: inout std_logic;  
	pin63	: inout std_logic;
	pin64	: inout std_logic;
	pin67	: inout std_logic;    
	pin68	: inout std_logic;
	pin69	: inout std_logic;
	pin70	: inout std_logic;
	pin71	: inout std_logic;
	pin72	: inout std_logic;
	pin73	: inout std_logic;
	pin74	: inout std_logic;
	pin75	: inout std_logic;
	pin76	: inout std_logic;
	pin77	: inout std_logic;
	pin78	: in std_logic;
	pin79	: in std_logic;
	pin80	: in std_logic;
	pin83	: in std_logic;
	pin84	: in std_logic);
end C346;
