--
--	Copyright Cypress Semiconductor Corporation, 1994,1995,1996
--	  as an unpublished work.
--

-- package int_math
--
-- Bit Vector support package:
--        warp -a work libbv
--
-- Contains these functions:

USE WORK.bv_math.ALL;
PACKAGE int_math IS
    FUNCTION bv2i   (bv	: BIT_VECTOR)			RETURN INTEGER;
    FUNCTION i2bv   (VAL, width	: INTEGER)		RETURN BIT_VECTOR;
    FUNCTION i2bvd  (VAL, width	: INTEGER)		RETURN BIT_VECTOR;
    FUNCTION "="    (a	: BIT_VECTOR; b	: INTEGER)	RETURN BOOLEAN;
    FUNCTION "/="   (a	: BIT_VECTOR; b	: INTEGER)	RETURN BOOLEAN;
    FUNCTION ">"    (a	: BIT_VECTOR; b	: INTEGER)	RETURN BOOLEAN;
    FUNCTION "<"    (a	: BIT_VECTOR; b	: INTEGER)	RETURN BOOLEAN;
    FUNCTION ">="   (a	: BIT_VECTOR; b	: INTEGER)	RETURN BOOLEAN;
    FUNCTION "<="   (a	: BIT_VECTOR; b	: INTEGER)	RETURN BOOLEAN;
    FUNCTION "+"    (a	: BIT_VECTOR; b	: INTEGER)	RETURN BIT_VECTOR;
    FUNCTION "-"    (a	: BIT_VECTOR; b	: INTEGER)	RETURN BIT_VECTOR;
END int_math;

PACKAGE BODY int_math IS

    -- bv2i
    --	Bit_vector to Integer.
    --	In:	bit_vector.
    --	Return:	integer.
    --
    FUNCTION BV2I (bv : Bit_vector) RETURN integer IS
      VARIABLE result, abit : integer range 0 to 2**bv'length - 1 := 0;
      VARIABLE  count            : integer := 0;
    BEGIN  --  BV2I 
      bits : FOR I IN bv'low to bv'high LOOP
      abit := 0;
      IF ((bv(I) = '1')) THEN
        abit := 2**(I - bv'low);
      END IF;
        result := result + abit;                --  Add in bit if '1'.
      count := count + 1;
      EXIT bits WHEN count = 32;         --  32 bits max.
      END LOOP bits;
      RETURN (result);
  
    END BV2I;

    -- i2bv
    --	Integer to Bit_vector.
    --	In:	integer, value and width.
    --	Return:	bit_vector, with right bit the most significant.
    --
    FUNCTION i2bv (val, width : integer) RETURN bit_vector is
	variable result : bit_vector(0 to width-1) := (others => '0');
	variable b : integer;
	variable bits : integer := width;
    begin
	if (bits > 32) then 
	    bits := 32;
	    assert (false)
	    report "TO_VECTOR: Widths > 32 not allowed, vector truncated"
	    severity warning;
	end if;
	if (bits <= 31) then
	    for i in result'reverse_range loop
		if ((val/(2**i)) mod 2) = 1 then
		    result(i) := '1';
		end if;
	    end loop;
	end if;
	if (bits = 32) then
	    b := val/2;
	    if ((val mod 2) = 1) then
		result(result'low) := '1';
	    end if;
	    for i in result'low to (result'high - 1) loop
		if ((b/(2**i)) mod 2) = 1 then
		    result(i+1) := '1';
		end if;
	    end loop;
	end if;
	if (2**(bits-2) < val/4) then
	    assert ((2**(bits-2)) > (val/4))
	    report "Value too big FOR BIT_VECTOR width"
	    severity warning;
	end if;
	return result;
    end i2bv;
 
    -- i2bvd
    --	Integer to Bit_vector.
    --	In:	integer, value and width.
    --	Return:	bit_vector, with left bit the most significant.
    --
    FUNCTION i2bvd  (VAL, width	: INTEGER)		RETURN BIT_VECTOR IS
	VARIABLE result : BIT_VECTOR (0 to width-1);
	VARIABLE resultd : BIT_VECTOR (width-1 downto 0) := (OTHERS=>'0');
    BEGIN
      result := i2bv(val, width);
      FOR i IN 0 TO width - 1 LOOP       --  Make result descending.
          resultd(i) := result(i);
      END LOOP;

      RETURN (resultd);
    END i2bvd;
    
    -- "="
    --	Overload "=" for bit_vector vs. integer.
    --	In:	bit_vector and integer.
    --	Return:	boolean.
    --
    FUNCTION "="    (a	: BIT_VECTOR; b	: INTEGER)	RETURN BOOLEAN IS
    BEGIN
	RETURN (bv2i(a) = b);
    END "=";
 
    -- "/="
    --	Overload "/=" for bit_vector vs. integer.
    --	In:	bit_vector and integer.
    --	Return:	boolean.
    --
    FUNCTION "/="   (a	: BIT_VECTOR; b	: INTEGER)	RETURN BOOLEAN IS
    BEGIN
	RETURN (bv2i(a) /= b);
    END "/=";
 
    -- ">"
    --	Overload ">" for bit_vector vs. integer.
    --	In:	bit_vector and integer.
    --	Return:	boolean.
    --
    FUNCTION ">"    (a	: BIT_VECTOR; b	: INTEGER)	RETURN BOOLEAN IS
    BEGIN
	RETURN (bv2i(a) > b);
    END ">";
 
    -- "<"
    --	Overload "<" for bit_vector vs. integer.
    --	In:	bit_vector and integer.
    --	Return:	boolean.
    --
    FUNCTION "<"    (a	: BIT_VECTOR; b	: INTEGER)	RETURN BOOLEAN IS
    BEGIN
	RETURN (bv2i(a) < b);
    END "<";
 
    -- ">="
    --	Overload ">=" for bit_vector vs. integer.
    --	In:	bit_vector and integer.
    --	Return:	boolean.
    --
    FUNCTION ">="   (a	: BIT_VECTOR; b	: INTEGER)	RETURN BOOLEAN IS
    BEGIN
	RETURN (bv2i(a) >= b);
    END ">=";
 
    -- "<="
    --	Overload "<=" for bit_vector vs. integer.
    --	In:	bit_vector and integer.
    --	Return:	boolean.
    --
    FUNCTION "<="   (a	: BIT_VECTOR; b	: INTEGER)	RETURN BOOLEAN IS
    BEGIN
	RETURN (bv2i(a) <= b);
    END "<=";
 
    -- "+"
    --	Overload "+" for bit_vector plus integer.
    --	In:	bit_vector and integer.
    --	Return:	bit_vector.
    --
    FUNCTION "+"    (a	: BIT_VECTOR; b	: INTEGER)	RETURN BIT_VECTOR IS
    BEGIN
	RETURN (a + i2bv(b, a'LENGTH));
    END "+";
 
    -- "-"
    --	Overload "-" for bit_vector plus integer.
    --	In:	bit_vector and integer.
    --	Return:	bit_vector.
    --
    FUNCTION "-"    (a	: BIT_VECTOR; b	: INTEGER)	RETURN BIT_VECTOR IS
    BEGIN
	RETURN (a - I2BV(b, a'LENGTH));
    END "-";
 
END int_math;
